/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * GeneralInfoNoteBookTab.cc
 * Copyright (C)  2007 Fernando Tarín Morales
 * icemanf@gmail.com 
 *
 * GeneralInfoNoteBookTab.cc is free software.
 * 
 * You may redistribute it and/or modify it under the terms of the
 * GNU General Public License, as published by the Free Software
 * Foundation; either version 3 of the License, or (at your option)
 * any later version.
 * 
 * GeneralInfoNoteBookTab.cc is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with GeneralInfoNoteBookTab.cc.  If not, write to:
 * 	The Free Software Foundation, Inc.,
 * 	51 Franklin Street, Fifth Floor
 * 	Boston, MA  02110-1301, USA.
 */

#include "GeneralInfoNoteBookTab.h"

#include "Lince/Core.h"
#include "Lince/TorrentsManager.h"
#include "Lince/Utils.h"

#include "../config.h"
#include <glibmm/i18n.h>

#include <sstream>

using namespace Lince;

GraphicBar::GraphicBar()
{
	currentPieces = 0;
	totalPieces = 1;
	percentageMode = false;
}

bool GraphicBar::on_expose_event(GdkEventExpose* event)
{
	// This is where we draw on the window
	Glib::RefPtr<Gdk::Window> window = get_window();
	if(window)
	{
		Gtk::Allocation allocation = get_allocation();
		
		const int width = allocation.get_width();
		const int height = allocation.get_height();
		double inc_x;
		
		Cairo::RefPtr<Cairo::Context> cr = window->create_cairo_context();
		
		// clip to the area indicated by the expose event so that we only redraw
		// the portion of the window that needs to be redrawn
		cr->rectangle(event->area.x, event->area.y, event->area.width, event->area.height);
		cr->clip();
		
		// Paint background in white.
		cr->set_source_rgb(1.0, 1.0, 1.0);
		
		cr->move_to(0, 0);
		cr->line_to(0, height);
		cr->line_to(width, height);
		cr->line_to(width, 0);
		cr->line_to(0, 0);
		
		cr->close_path();		
		cr->fill_preserve();
		
		cr->stroke();
		
		// Paint black border
		cr->set_line_width(0.75);
		cr->set_source_rgb(0.0, 0.0, 0.0);
		
		cr->move_to(0, 0);
		cr->line_to(0, height);
		cr->line_to(width, height);
		cr->line_to(width, 0);
		cr->line_to(0, 0);
		
		cr->stroke();
		
		// Draw the blue bars
		cr->set_source_rgb(0.0, 0.5, 1.0);
		
		// Use as percentage bars
		if (percentageMode)
		{
			cr->move_to(0, 0);
			cr->line_to(0, height);
			cr->line_to((width * percentage), height);
			cr->line_to((width * percentage), 0);
			cr->close_path();		
			cr->fill_preserve();
			
			return true;
		}
		
		if (currentPieces == totalPieces)
		{
			cr->move_to(0, 0);
			cr->line_to(0, height);
			cr->line_to(width, height);
			cr->line_to(width, 0);
			cr->close_path();		
			cr->fill_preserve();
		}
		else
		{
			inc_x = width / (double) totalPieces;
			cr->set_line_width(inc_x);
			
			int i, lines = 0;
			libtorrent::bitfield::const_iterator j;
			
			for (i=0, j=values.begin(); j!=values.end(); ++j, i++)
			{
			//for (i=0; i<numPieces; i++){
				if (lines > currentPieces) break;
				if (*j)
				{
					double temp_x = i * inc_x;
					cr->move_to(temp_x, 0);
					cr->line_to(temp_x, height);
					lines++;
				}
			}
		}
		
		cr->stroke();
	}
	
	return true;
}

bool GraphicBar::redraw()
{
	Glib::RefPtr<Gdk::Window> win = get_window();
    if (win)
    {
        Gdk::Rectangle r(0, 0, get_allocation().get_width(),
                get_allocation().get_height());
        win->invalidate_rect(r, false);
    }
    return true;
}

GraphicBar::~GraphicBar()
{
}

GeneralInfoNoteBookTab::GeneralInfoNoteBookTab():
	nameTab(_("General")),
	downloadedLabel(_("Downloaded")),
	downloadedPercentageLabel("0.0%"),
	availabilityLabel(_("Availability")),
	availabilityValue("0.0"),
	nameLabel(_("Name:")),
	storeLabel(_("Stored at:")),
	partsLabel(_("Parts:")),
	trackerLabel("Tracker:"),
	updateLabel(_("Update:")),
	statusLabel(_("Status:")),
	sizeLabel(_("Size:")),
	hashLabel("Hash:"),
	partSizeLabel(_("Size part:")),
	createdLabel(_("Created:")),
	commentLabel(_("Comment: ")),
	timeLabel(_("Time: ")),
	restLabel(_("Rest: ")),
	hashsLabel(_("Failed Hashes: ")),
	dlLabel(_("Downloaded: ")),
	downSpeedLabel(_("Down. Speed: ")),
	seedersLabel(_("Seeders: ")),
	sentLabel(_("Sent: ")),
	upSpeedLabel(_("Up. Speed: ")),
	clientsLabel(_("Clients: "))
{
		
	Gtk::Table *downTable = Gtk::manage(new Gtk::Table(2, 3, false));
		
	// Downloaded Row
	gb.property_height_request() = 25;
	downloadedLabel.property_xalign() = 0.0;
	downTable->attach(downloadedLabel, 0, 1, 0, 1, Gtk::FILL, Gtk::SHRINK, 5, 5);
	downTable->attach(gb, 1, 2, 0, 1, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 15, 5);
	downTable->attach(downloadedPercentageLabel, 2, 3, 0, 1, Gtk::SHRINK, Gtk::SHRINK, 10, 5);
	
	
	// Availability Row
	availabilityLabel.property_xalign() = 0.0; availabilityValue.property_xalign() = 0.0;
	downTable->attach(availabilityLabel, 0, 1, 1, 2, Gtk::FILL, Gtk::SHRINK, 5, 5);
	downTable->attach(availabilityValue, 1, 2, 1, 2, Gtk::FILL, Gtk::SHRINK, 15, 5);
	
	// Transference Frame
	transferenceFrame.set_shadow_type(Gtk::SHADOW_OUT);
	transferenceFrame.set_label(_("Transference"));
		
	Gtk::Table *transTable = Gtk::manage(new Gtk::Table(3, 6, false));
	
	timeLabel.property_xalign() = 0; restLabel.property_xalign() = 0; hashsLabel.property_xalign() = 0;
	dlLabel.property_xalign() = 0; downSpeedLabel.property_xalign() = 0; upSpeedLabel.property_xalign() = 0;
	sentLabel.property_xalign() = 0; seedersLabel.property_xalign() = 0; clientsLabel.property_xalign() = 0;
		
	timeValueLabel.property_xalign() = 0; restValueLabel.property_xalign() = 0; hashsValueLabel.property_xalign() = 0;
	dlValueLabel.property_xalign() = 0; downSpeedValueLabel.property_xalign() = 0; upSpeedValueLabel.property_xalign() = 0;
	sentValueLabel.property_xalign() = 0; seedersValueLabel.property_xalign() = 0; clientsValueLabel.property_xalign() = 0;
	
	transTable->attach(timeLabel, 0, 1, 0, 1, Gtk::FILL, Gtk::SHRINK, 10, 5);
	transTable->attach(timeValueLabel, 1, 2, 0, 1, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
	transTable->attach(restLabel, 2, 3, 0, 1, Gtk::FILL, Gtk::SHRINK, 10, 5);
	transTable->attach(restValueLabel, 3, 4, 0, 1, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
	transTable->attach(hashsLabel, 4, 5, 0, 1, Gtk::FILL, Gtk::SHRINK, 10, 5);
	transTable->attach(hashsValueLabel, 5, 6, 0, 1, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
	
	transTable->attach(dlLabel, 0, 1, 1, 2, Gtk::FILL, Gtk::SHRINK, 10, 5);
	transTable->attach(dlValueLabel, 1, 2, 1, 2, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
	transTable->attach(downSpeedLabel, 2, 3, 1, 2, Gtk::FILL, Gtk::SHRINK, 10, 5);
	transTable->attach(downSpeedValueLabel, 3, 4, 1, 2, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
	transTable->attach(seedersLabel, 4, 5, 1, 2, Gtk::FILL, Gtk::SHRINK, 10, 5);
	transTable->attach(seedersValueLabel, 5, 6, 1, 2, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
		
	transTable->attach(sentLabel, 0, 1, 2, 3, Gtk::FILL, Gtk::SHRINK, 10, 5);
	transTable->attach(sentValueLabel, 1, 2, 2, 3, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
	transTable->attach(upSpeedLabel, 2, 3, 2, 3, Gtk::FILL, Gtk::SHRINK, 10, 5);
	transTable->attach(upSpeedValueLabel, 3, 4, 2, 3, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
	transTable->attach(clientsLabel, 4, 5, 2, 3, Gtk::FILL, Gtk::SHRINK, 10, 5);
	transTable->attach(clientsValueLabel, 5, 6, 2, 3, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
	
	transferenceFrame.add(*transTable);
	
	// Information Frame
	informationFrame.set_shadow_type(Gtk::SHADOW_OUT);
	informationFrame.set_label(_("Information"));
	
	nameLabel.property_xalign()  = 0; statusLabel.property_xalign() = 0;
	storeLabel.property_xalign() = 0; sizeLabel.property_xalign() = 0;
	partsLabel.property_xalign() = 0; hashLabel.property_xalign() = 0;
	trackerLabel.property_xalign() = 0; partSizeLabel.property_xalign() = 0;
	updateLabel.property_xalign()  = 0; createdLabel.property_xalign() = 0;
	commentLabel.property_xalign() = 0;
		
	nameValueLabel.property_xalign()  = 0; statusValueLabel.property_xalign() = 0;
	storeValueLabel.property_xalign() = 0; sizeValueLabel.property_xalign() = 0;
	partsValueLabel.property_xalign() = 0; hashValueLabel.property_xalign() = 0;
	trackerValueLabel.property_xalign() = 0; partSizeValueLabel.property_xalign() = 0;
	updateValueLabel.property_xalign()  = 0; createdValueLabel.property_xalign() = 0;
	
	nameValueLabel.set_selectable(); storeValueLabel.set_selectable(); //trackerLabel.set_selectable();
	hashValueLabel.set_selectable(); createdValueLabel.set_selectable(); commentLabel.set_selectable();
	
	/*nameLabel.property_can_focus() = false; storeLabel.property_can_focus() = false;
	hashLabel.property_can_focus() = false; createdLabel.property_can_focus() = false;
	commentLabel.property_can_focus() = false;*/
	
	storeValueLabel.set_ellipsize(Pango::ELLIPSIZE_END);
	hashValueLabel.set_ellipsize(Pango::ELLIPSIZE_END); 
	commentLabel.set_ellipsize(Pango::ELLIPSIZE_END);
	//trackerLabel.set_ellipsize(Pango::ELLIPSIZE_END); 
	nameValueLabel.set_ellipsize(Pango::ELLIPSIZE_END);
		
	Gtk::Table *infoTable = Gtk::manage(new Gtk::Table(5, 4, false));
	
	infoTable->attach(nameLabel, 0, 1, 0, 1, Gtk::FILL, Gtk::SHRINK, 10, 5);
	infoTable->attach(nameValueLabel, 1, 2, 0, 1, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
	infoTable->attach(statusLabel, 2, 3, 0, 1, Gtk::FILL, Gtk::SHRINK, 10, 5);
	infoTable->attach(statusValueLabel, 3, 4, 0, 1, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
	
	infoTable->attach(storeLabel, 0, 1, 1, 2, Gtk::FILL, Gtk::SHRINK, 10, 5);
	infoTable->attach(storeValueLabel, 1, 2, 1, 2, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
	infoTable->attach(sizeLabel, 2, 3, 1, 2, Gtk::FILL, Gtk::SHRINK, 10, 5);
	infoTable->attach(sizeValueLabel, 3, 4, 1, 2, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
		
	infoTable->attach(partsLabel, 0, 1, 2, 3, Gtk::FILL, Gtk::SHRINK, 10, 5);
	infoTable->attach(partsValueLabel, 1, 2, 2, 3, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
	infoTable->attach(hashLabel, 2, 3, 2, 3, Gtk::FILL, Gtk::SHRINK, 10, 5);
	infoTable->attach(hashValueLabel, 3, 4, 2, 3, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
		
	infoTable->attach(trackerLabel, 0, 1, 3, 4, Gtk::FILL, Gtk::SHRINK, 10, 5);
	infoTable->attach(trackerValueLabel, 1, 2, 3, 4, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
	infoTable->attach(partSizeLabel, 2, 3, 3, 4, Gtk::FILL, Gtk::SHRINK, 10, 5);
	infoTable->attach(partSizeValueLabel, 3, 4, 3, 4, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
		
	infoTable->attach(updateLabel, 0, 1, 4, 5, Gtk::FILL, Gtk::SHRINK, 10, 5);
	infoTable->attach(updateValueLabel, 1, 2, 4, 5, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
	infoTable->attach(createdLabel, 2, 3, 4, 5, Gtk::FILL, Gtk::SHRINK, 10, 5);
	infoTable->attach(createdValueLabel, 3, 4, 4, 5, (Gtk::FILL | Gtk::EXPAND), Gtk::SHRINK, 10, 5);
	
	commentRow.pack_start(commentLabel, Gtk::PACK_EXPAND_WIDGET, 10);
	
	infoFrameBox.pack_start(*infoTable, Gtk::PACK_SHRINK, 5);
	infoFrameBox.pack_start(commentRow, Gtk::PACK_SHRINK, 0);
	informationFrame.add(infoFrameBox);
	
	vbox.pack_start(*downTable, Gtk::PACK_SHRINK, 5);
	vbox.pack_start(transferenceFrame, Gtk::PACK_SHRINK, 5);
	vbox.pack_start(informationFrame, Gtk::PACK_EXPAND_WIDGET, 5);
	
	std::list<Gtk::Widget *> focus_order;
	infoFrameBox.set_focus_chain(focus_order);
	
	isStaticDataAdded = false;
	downloadedPieces = -1;
	
}

void GeneralInfoNoteBookTab::updateGeneralInfoNoteBookTab(Torrent *t)
{
	std::ostringstream tempValue;
	
	if (!isStaticDataAdded)
	{
		isStaticDataAdded = true;
		addStaticData(t);
	}
	
	// Transference
	if (t->get_status() != libtorrent::torrent_status::checking_files && 
		t->get_status() != libtorrent::torrent_status::queued_for_checking)
	{
		if (downloadedPieces != t->get_downloaded_pieces())
		{
			downloadedPieces = t->get_downloaded_pieces();
			
			gb.setCurrentPieces(t->get_downloaded_pieces());
			gb.setTotalPieces(t->get_total_pieces());
			gb.setValues(t->get_pieces());
			gb.redraw();
			
			if (t->get_progress() != 1){
				tempValue.setf(std::ios::showpoint | std::ios::fixed);
				tempValue << std::setprecision(2) << t->get_percentage() << "%";
			}
			else
				tempValue << "100%";
			
			downloadedPercentageLabel.set_label(tempValue.str());
		}
		
		tempValue.str("");
		if (t->get_progress() == 1)
		{
			tempValue << _("Complete");
		}
		else
		{
			if (t->get_eta() != FLT_MAX)
				tempValue << time_to_string(static_cast<boost::int64_t>(t->get_eta()))
					<< " " << size_to_string(t->get_remaining());
			else
				tempValue << size_to_string(t->get_remaining());
		}
		restValueLabel.set_label(tempValue.str());
	}
	else
	{
		gb.setPercentage(t->get_progress());
		gb.redraw();
		
		tempValue.setf(std::ios::showpoint | std::ios::fixed);
		tempValue << std::setprecision(2) << t->get_percentage() << "%";
		downloadedPercentageLabel.set_label(tempValue.str());
		
		tempValue.str("");
		tempValue << size_to_string(t->get_size() - (t->get_size() * t->get_progress()));
		restValueLabel.set_label(tempValue.str());
	}
		
	
	tempValue.str("");
	if (t->get_distributed_copies() != -1)
	{
		tempValue.setf(std::ios::showpoint | std::ios::fixed);
		tempValue << std::setprecision(2) << t->get_distributed_copies();
	}
	else
		tempValue << "-";
	availabilityValue.set_label(tempValue.str());
	
	timeValueLabel.set_label(time_to_string(t->elapsed_time()));
	
	dlValueLabel.set_label(size_to_string(t->get_downloaded()));
	
	tempValue.str("");
	tempValue << t->failed_hashes() << " (" << size_to_string (t->failed_hashes()
			* t->get_piece_length())  << ")";
	hashsValueLabel.set_label(tempValue.str());
	
	sentValueLabel.set_label(size_to_string(t->total_uploaded()));
	
	downSpeedValueLabel.set_label(size_to_string(t->get_download_speed()));
	
	upSpeedValueLabel.set_label(size_to_string(t->get_upload_speed()));
	
	tempValue.str("");
	tempValue << t->get_seeders();
	if ( t->get_total_seeders() != -1)
		tempValue << " (" << t->get_total_seeders() << ")";
	seedersValueLabel.set_label(tempValue.str());
	
	tempValue.str("");
	tempValue << t->get_downloaders();
	if ( t->get_total_downloaders() != -1)
		tempValue << " (" << t->get_total_downloaders() << ")";
	clientsValueLabel.set_label(tempValue.str());
	
	// Information
	statusValueLabel.set_label(t->get_status_string());
	
	tempValue.str("");
	tempValue << t->get_downloaded_pieces() << "/" << t->get_total_pieces();
 	partsValueLabel.set_label(tempValue.str());
	
	// trackerValueLabel.set_label(t->trackers().begin()->url);
	if (t->status().current_tracker.empty())
		trackerValueLabel.set_label(_("Not connected"));
	else
		trackerValueLabel.set_label(t->status().current_tracker);
	
	tempValue.str("");
	tempValue << t->get_next_announce();
	updateValueLabel.set_label(tempValue.str());
}

void GeneralInfoNoteBookTab::addStaticData(Torrent *t)
{
	std::ostringstream tempValue;
	
	// Static values
	nameValueLabel.set_label(t->name());
			
	sizeValueLabel.set_label(size_to_string(t->get_size()));
		
	if (t->get_torrent_info().num_files() == 1)
		storeValueLabel.set_label(Glib::build_filename(t->save_path().native_directory_string(),
					t->get_torrent_info().file_at(0).path.string()));
	else
		storeValueLabel.set_label((Glib::build_filename(
					t->save_path().native_directory_string(), t->name())));
	
	tempValue << t->get_hash();
	hashValueLabel.set_label(tempValue.str());
		
	partSizeValueLabel.set_label(size_to_string(t->get_piece_length()));
		
	createdValueLabel.set_label(t->get_date_as_string());
		
	tempValue.str("");
	tempValue << _("Comment: ") << t->get_comment();
	commentLabel.set_label(tempValue.str());
}

GeneralInfoNoteBookTab::~GeneralInfoNoteBookTab(){
}
