//           LogTab.cc
//  Mon Aug 13 12:42:20 2007
//  Copyright  2007  Fernando Tarín Morales
//  icemanf@gmail.com

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA


#include "BaseTab.h"
#include "LogTab.h"

#include <glibmm/i18n.h>

#include "Lince/Core.h"
#include "Lince/AlertsManager.h"
#include "Lince/Utils.h"

#include <boost/date_time/posix_time/posix_time.hpp>
#include <boost/date_time/gregorian/gregorian.hpp>
#include <boost/bind.hpp>

#if !defined(_WIN32) && defined(__MINGW__)
#include <time.h>
#endif

#include <iostream>

using namespace Lince;

LogTab::LogTab():
	BaseTab(_("Log"))
{
	//Add the TreeView, inside a ScrolledWindow, with the button underneath:
	logScrolledWindow.add(logTextView);

	//Only show the scrollbars when they are necessary:
	logScrolledWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	refTagTable = Gtk::TextBuffer::TagTable::create();
	logTextBuffer = Gtk::TextBuffer::create(refTagTable);

	// TextView properties
	logTextView.set_buffer(logTextBuffer);
	logTextView.set_cursor_visible (false);
	logTextView.set_wrap_mode(Gtk::WRAP_WORD);
	logTextView.set_editable(false);

	logFrame.set_label(_("Log Data"));
	logFrame.add(logScrolledWindow);


	maxLogSize = 524288; // 512kB
	saveToFile = useColoredOutput = false;
	autoScroll = isActive = true;

	// Tags
	Glib::RefPtr<Gtk::TextBuffer::Tag> refTagMatch =
		Gtk::TextBuffer::Tag::create("GrayBold");

	refTagMatch->property_weight() = Pango::WEIGHT_BOLD;
	refTagMatch->property_foreground() = "gray";
	refTagTable->add(refTagMatch);

	refTagMatch = Gtk::TextBuffer::Tag::create("Bold");
	refTagMatch->property_weight() = Pango::WEIGHT_BOLD;
	refTagTable->add(refTagMatch);

	refTagMatch = Gtk::TextBuffer::Tag::create("Red");
	//refTagMatch->property_foreground() = "red";
	refTagMatch->property_foreground() = "#dd0000";
	refTagTable->add(refTagMatch);

	refTagMatch = Gtk::TextBuffer::Tag::create("Blue");
	//refTagMatch->property_foreground() = "blue";
	refTagMatch->property_foreground() = "#0000dd";
	refTagTable->add(refTagMatch);

	refTagMatch = Gtk::TextBuffer::Tag::create("Green");
	//refTagMatch->property_foreground() = "green";
	refTagMatch->property_foreground() = "#338833";
	refTagTable->add(refTagMatch);

	refTagMatch = Gtk::TextBuffer::Tag::create("Orange");
	refTagMatch->property_foreground() = "orange";
	refTagTable->add(refTagMatch);

	// Add Start Message.
	Glib::ustring now = getTime();

	appendGrayBoldText(now);
	appendBoldText(std::string(PACKAGE) + std::string(_(" started\n")));

	// Open Log File
	fileName = Glib::build_filename(get_base_directory().native_directory_string(),
									std::string("session.log"));

	fileStream.open(std::string(fileName.c_str()));

	// Bind functions
	Core::get_alerts_manager()->finished_torrent_signal_log(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->listen_failed_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->listen_succeded_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->portmap_failed_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->portmap_success_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->file_error_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->tracker_failed_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->tracker_reply_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->tracker_warning_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->http_seed_failed_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->hash_failed_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->peer_banned_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->fastresume_rejected_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->storage_moved_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->torrent_checked_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->torrent_added_log_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->torrent_removed_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->torrent_queued_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->torrent_unqueued_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->torrent_stopped_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->torrent_started_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_session_manager()->get_ip_filter()->ip_banned_signal(
			boost::bind(&LogTab::appendMessage, this, _1, WARNING));

	Core::get_session_manager()->get_ip_filter()->ip_unbanned_signal(
			boost::bind(&LogTab::appendMessage, this, _1, WARNING));

	Core::get_alerts_manager()->magnet_added_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->metadata_recived_log_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));

	Core::get_alerts_manager()->metadata_failed_signal(
			boost::bind(&LogTab::appendMessage, this, _1, _2));
}

LogTab::~LogTab()
{
	if (saveToFile)
		saveCurrentBufferToFile();

	if (fileStream)
		fileStream.close();
}

void LogTab::setSaveToFile(bool value)
{
	saveToFile = value;

	if (value)
		if (!fileStream) fileStream.open(std::string(fileName.c_str()));
	else
		if (fileStream) fileStream.close();
}

void LogTab::saveCurrentBufferToFile()
{
	if (fileStream)
		fileStream << logTextBuffer->get_text();
}

void LogTab::appendText(const Glib::ustring & message)
{
	logTextBuffer->insert(logTextBuffer->get_iter_at_offset
						  (logTextBuffer->get_char_count()), message);
}

void LogTab::appendBoldText(const Glib::ustring & message)
{
	logTextBuffer->insert_with_tag(logTextBuffer->get_iter_at_offset
		(logTextBuffer->get_char_count()), message, refTagTable->lookup("Bold"));
}

void LogTab::appendGrayBoldText(const Glib::ustring & message)
{
	if ((logTextBuffer->size() + message.size()) >= maxLogSize)
	{
		saveCurrentBufferToFile();
		cleanLogTab();
	}

	logTextBuffer->insert_with_tag(logTextBuffer->get_iter_at_offset
		(logTextBuffer->get_char_count()), message, refTagTable->lookup("GrayBold"));

	if (autoScroll)
	{
		Glib::RefPtr<Gtk::TextBuffer::Mark> mark = logTextBuffer->get_insert();
		logTextView.scroll_to(mark, 0.0);
	}
}

void LogTab::appendRedText(const Glib::ustring & message)
{
	logTextBuffer->insert_with_tag(logTextBuffer->get_iter_at_offset
			(logTextBuffer->get_char_count()), message, refTagTable->lookup("Red"));
}

void LogTab::appendBlueText(const Glib::ustring & message)
{
	logTextBuffer->insert_with_tag(logTextBuffer->get_iter_at_offset
			(logTextBuffer->get_char_count()), message, refTagTable->lookup("Blue"));
}

void LogTab::appendGreenText(const Glib::ustring & message)
{
	logTextBuffer->insert_with_tag(logTextBuffer->get_iter_at_offset
		(logTextBuffer->get_char_count()), message, refTagTable->lookup("Green"));
}

void LogTab::appendOrangeText(const Glib::ustring & message)
{
	logTextBuffer->insert_with_tag(logTextBuffer->get_iter_at_offset
		(logTextBuffer->get_char_count()), message, refTagTable->lookup("Orange"));
}

Glib::ustring LogTab::getTime()
{
#if defined(_WIN32) && defined(__MINGW32__)
	std::string now = boost::posix_time::to_simple_string
		(boost::posix_time::second_clock::local_time()) + std::string(": ");
	return Glib::ustring(now);
#else
	char date[30];
	struct tm time_tm;
	time_t time_t; time(&time_t);
	localtime_r(&time_t, &time_tm);
	strftime(date, 30, "%Y-%b-%d %H:%M:%S", &time_tm);

	Glib::ustring now = Glib::ustring(date) + Glib::ustring(": ");
	return now;
#endif
}

void LogTab::logMessage(struct message& m)
{

	// Append Message
	appendGrayBoldText(m.time);

	if (!useColoredOutput)
		appendText(m.message);
	else
	{
		if (m.level == INFO)
			appendGreenText(m.message);
		else if (m.level == FATAL)
			appendRedText(m.message);
		else if (m.level == WARNING)
			appendBlueText(m.message);
		else
			appendText(m.message);
	}
	appendText(Glib::ustring("\n"));

}

void LogTab::appendMessage(std::string message, int level)
{
	// Check if we have to store the message
	if (!isActive || level > logLevel) return;

	// Get the time and the utf8 version of the message
	Glib::ustring now = getTime();
	Glib::ustring utf8_message;

	utf8_message.assign(message);
	if (!utf8_message.validate())
		utf8_message.assign(Glib::locale_to_utf8(message));

	// Store the message in the stack
	struct message m = {now, utf8_message, level};
	messages.push(m);
}

void LogTab::flushMessages(void)
{
	// Check that we have some messages
	if (messages.empty())
		return;

	// flush 10 messages at max
	for (int i=0; i<10; i++)
	{
		// Log the message
		logMessage(messages.front());
		messages.pop();

		// We are done if we don't have more messages
		if (messages.empty())
			break;
	}
}

void LogTab::cleanLogTab(void)
{
	if (logTextBuffer->get_char_count())
	{
		Gtk::TextIter begin, end;
		logTextBuffer->get_bounds(begin, end);
		logTextBuffer->erase(begin, end);
	}
}
