/***************************************************************************
 *            OptionsTab.h
 *
 *  Sat Mar 31 01:24:27 2007
 *  Copyright  2007  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#ifndef _OPTIONSTAB_H
#define _OPTIONSTAB_H

#include <gtkmm/widget.h>
#include <gtkmm/box.h>
#include <gtkmm/paned.h>
#include <gtkmm/treestore.h>
#include <gtkmm/treeview.h>

#include "BaseTab.h"
#include "GuiSetupManager.h"
#include "Utils.h"

#include "OptionsWindows.h"

class NoteBookManager;

class OptionsTab: public BaseTab
{
	private:
	   	// Panes.
		Gtk::HPaned pane;

		// Rigth pane
		enum configWindows { FIRST_WINDOW, CONNECTION=FIRST_WINDOW, PROXY,
			ENCRYPTION, TRANSFERENCE, SCHEDULER, FILES, TORRENTS, STATISTICS,
			LOG, NOTIFICATIONS, QUEUE, IP_FILTER, SEARCH, LAST_WINDOW };

		Gtk::HBox configWindowsBox;
		bool windowsUsed[LAST_WINDOW];

		void emptyWindowsBox();

		class ConnectionWindow cw;
		class TransferenceWindow tw;
		class ProxyWindow pw;
		class EncryptionWindow ew;
		class FilesWindow fw;
		class TorrentsOptionsWindow tow;
		class StatisticsWindow sw;
		class LogWindow lw;
		class NotificationsWindow nw;
		class QueueWindow qw;
		class IpFilterWindow ifw;
		class SearchWindow shw;
		class SchedulerWindow scw;

		// Left's pane
		Gtk::ScrolledWindow optionsWindow;
		Utils::ModelColumnsIntString m_Columns;
		Gtk::TreeView optionsTreeView;
		Glib::RefPtr<Gtk::TreeStore> refTreeModel;

		// Pointer to Setup Manager.
		GuiSetupManager * gsm;

		// Pointer to NoteBookManager
		NoteBookManager * nb;

		// Signal handerls
		virtual void on_treeview_row_changed();

		void updateOptionsTab();
		void banIP(const std::string &);

	public:
		OptionsTab(NoteBookManager *);
		~OptionsTab();

		void setConnectionWindow();
		void setGuiSetupManager(GuiSetupManager * sm );
		virtual Gtk::Widget * getTabChild(void) { return & pane; };

		// Config values
		// Connection Window
		void setFirstTcpButtonConfigValue();
		void setLastTcpButtonConfigValue();
		void setDhtConfigValue();
		void setUpnpConfigValue();
		void setLocalDiscoveryConfigValue();
		void setMetadataConfigValue();
		void setPexConfigValue();
		void setSmartBanValue();
		void setAdvancedBindToInterfaceConfigValue();
		void setAdvancedMaxHalfOpenConnectionsConfigValue();

		// EncryptionWindow
		void setEncryptionEnableButtonConfigValue();
		void setPreferRC4ConfigValue();
		void setInconmingConnectionsButtonConfigValue();
		void setOutgoingConnectionsButtonConfigValue();
		void setAllowedEncryptionButtonConfigValue();

		// Proxy Window
		void setServerProxyNameConfigValue();
		void setServerProxyPortConfigValue();
		void setServerProxyLoginConfigValue();
		void setServerProxyPassConfigValue();
		void setServerProxyEnableCheckButtonConfigValue();
		void setServerProxyTypeConfigValue();

		void setClientProxyNameConfigValue();
		void setClientProxyPortConfigValue();
		void setClientProxyLoginConfigValue();
		void setClientProxyPassConfigValue();
		void setClientProxyEnableCheckButtonConfigValue();
		void setClientProxyTypeConfigValue();

		// Transference Window
		void setMaxUploadButtonConfigValue();
		void setMaxDownloadButtonConfigValue();
		void setMaxConnectsButtonConfigValue();
		void setMaxUploadSlotsButtonConfigValue();
		void setMaxUploadCheckButtonConfigValue();
		void setMaxDownloadCheckButtonConfigValue();
		void setMaxConnectsCheckButtonConfigValue();
		void setMaxUploadSlotsCheckButtonConfigValue();
		void setShareRatioConfigValue();
		void setShareRatioCheckButtonConfigValue();
		void setAllowConnectionsFromTheSameIpConfigValue();
		void setPrioritizeFirstPartOfFilesConfigValue();
		// To set speed from systray menu.
		void setMaxConfigUploadSpeed(int);
		void setMaxConfigDownloadSpeed(int);
		void setMaxConfigConnects(int);
		void setMaxConfigUploadSlots(int);

		// Files Window
		void setDirectoryEntryConfigValue();
		void setDownloadDefaultDirectoryCheckButtonConfigValue();
		void setStorageTypeConfigValue();
		void setFastResumeCheckButtonConfigValue();
		void setFastResumeTimeUpdateButtonConfigValue();
		void setConfirmDataEliminationCheckButtonConfigValue();
		void setAskWhenLoadingTorrentsUsingDbusRequestsConfigValue();

		// Torrents Window
		void setSaveTorrentsCheckButtonConfigValue();
		void setSaveDirectoryEntryConfigValue();
		void setRemoveOriginalTorrentConfigValue();
		void setRemoveTorrentWhenFinishedConfigValue();
		void setLoadTorrentsCheckButtonConfigValue();
		void setLoadDirectoryEntryConfigValue();
		void setAllowChangeDirectoryConfigValue();

		// Statistics Window
		void setStatisticsTabCheckButtonConfigValue();
		void setUpdateTimeButtonConfigValue();

		// LogWindow
		void setActivateLogCheckButtonConfigValue();
		void setColoredOutputCheckButtonConfigValue();
		void setSaveLogFileCheckButtonConfigValue();
		void setBufferSizeButtonConfigValue();
		void setAutoScrollCheckButtonConfigValue();
		void setLogLevelConfigValue();

		// NotificationsWindow
		void setActivateNotificationsConfigValue();
		void setNotifyFinishedTorrentsConfigValue();
		void setNotifyStoppedTorrentsConfigValue();
		void setNotifyStartedTorrentsConfigValue();
		void setNotifyAddedTorrentsConfigValue();
		void setNotifyRemovedTorrentsConfigValue();
		void setNotifyQueuedTorrentsConfigValue();
		void setNotifyUnqueuedTorrentsConfigValue();
		void setNotifyShowFromSystrayConfigValue();

		// QueueWindow
		void setActiveDownloadsConfigValue();
		void setMaxDownloadsConfigValue();
		void setActiveSharesConfigValue();

		// IpFilterWindow
		void setActivateIpFilterConfigValue();
		void updateIpFilterData() { ifw.updateIpFilterData(); };
		void selectIpFilterFile();
		void addIpFilter();
		void removeSelectedIpFilter();

		// SearchWindow
		void setBtjunkieConfigValue();
		void setIsohuntConfigValue();
		void setPirateBayConfigValue();
		void setMininovaConfigValue();
		void setTorrentreactorConfigValue();
		void setFilterResultsWithoutSeedersConfigValue();
		void setMaxSearchConfigValue();

		// SchedulerWindow
		void setUseSchedulerConfigValue();

		void updateSchedulerConfig();

		void setMondayConfigValue();
		void setTuesdayConfigValue();
		void setWednesdayConfigValue();
		void setThursdayConfigValue();
		void setFridayConfigValue();
		void setSaturdayConfigValue();
		void setSundayConfigValue();

		void setDownloadHour0ConfigValue();
		void setUploadHour0ConfigValue();
		void setDownloadHour1ConfigValue();
		void setUploadHour1ConfigValue();
		void setDownloadHour2ConfigValue();
		void setUploadHour2ConfigValue();
		void setDownloadHour3ConfigValue();
		void setUploadHour3ConfigValue();
		void setDownloadHour4ConfigValue();
		void setUploadHour4ConfigValue();
		void setDownloadHour5ConfigValue();
		void setUploadHour5ConfigValue();
		void setDownloadHour6ConfigValue();
		void setUploadHour6ConfigValue();
		void setDownloadHour7ConfigValue();
		void setUploadHour7ConfigValue();
		void setDownloadHour8ConfigValue();
		void setUploadHour8ConfigValue();
		void setDownloadHour9ConfigValue();
		void setUploadHour9ConfigValue();
		void setDownloadHour10ConfigValue();
		void setUploadHour10ConfigValue();
		void setDownloadHour11ConfigValue();
		void setUploadHour11ConfigValue();
		void setDownloadHour12ConfigValue();
		void setUploadHour12ConfigValue();
		void setDownloadHour13ConfigValue();
		void setUploadHour13ConfigValue();
		void setDownloadHour14ConfigValue();
		void setUploadHour14ConfigValue();
		void setDownloadHour15ConfigValue();
		void setUploadHour15ConfigValue();
		void setDownloadHour16ConfigValue();
		void setUploadHour16ConfigValue();
		void setDownloadHour17ConfigValue();
		void setUploadHour17ConfigValue();
		void setDownloadHour18ConfigValue();
		void setUploadHour18ConfigValue();
		void setDownloadHour19ConfigValue();
		void setUploadHour19ConfigValue();
		void setDownloadHour20ConfigValue();
		void setUploadHour20ConfigValue();
		void setDownloadHour21ConfigValue();
		void setUploadHour21ConfigValue();
		void setDownloadHour22ConfigValue();
		void setUploadHour22ConfigValue();
		void setDownloadHour23ConfigValue();
		void setUploadHour23ConfigValue();
};

#endif /* _OPTIONSTAB_H */
