//           OptionsWindows.cxx
//  Mon Apr  9 13:57:39 2007
//  Copyright  2007  Fernando Tarín Morales
//  icemanf@gmail.com

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA

#include "OptionsWindows.h"
#include "OptionsTab.h"

#include "../config.h"
#include <glibmm/i18n.h>

#include "Lince/Core.h"
#include "Lince/SessionManager.h"

using namespace Lince;

OptionsWindowsBase::OptionsWindowsBase(Glib::ustring title)
{
	Glib::ustring tmpString;

	//headerText("<span background=\"#6484a4\" foreground=\"#ffffff\"><big><big>Transference</big></big></span>"),
	tmpString = "<big><big>" + title + "</big></big>";
	headerLabelText.set_label(tmpString);
	headerLabelText.set_use_markup();
	headerLabelText.set_alignment(0.35, 0.5);
	//headerLabelText.set_padding(100, 0);

	headerText.modify_bg(Gtk::STATE_NORMAL, Gdk::Color("#6484a4"));
	headerText.modify_bg(Gtk::STATE_ACTIVE, Gdk::Color("#6484a4"));
	headerText.modify_bg(Gtk::STATE_PRELIGHT, Gdk::Color("#6484a4"));
	headerText.modify_bg(Gtk::STATE_SELECTED, Gdk::Color("#6484a4"));
	headerText.modify_bg(Gtk::STATE_INSENSITIVE, Gdk::Color("#6484a4"));
	headerText.add(headerLabelText);
};

OptionsWindowsBase::~OptionsWindowsBase()
{
}

ConnectionWindow::ConnectionWindow(OptionsTab *oT):
	OptionsWindowsBase(_("Connection")),
	tcpText1(_("Port range:")),
	tcpText2(_("to")),
	tcpRow(false, 5),
	dhtActivateButton(_("Add suppport for DHT.")),
	upnpActivateButton(_("Add suppport for UPNP.")),
	localDiscoveryActivateButton(_("Add support for searching peers in the local network.")),
	metadataActivateButton(_("Adds suppport for exchanging metadata from peers. (*)")),
	pexActivateButton(_("Adds support for uTorrent peer exchange. (*)")),
	smartBanButton(_("Adds support for smart ban plugin. (*)")),
	advancedBindToInterfaceLabel(_("(*) Bind to interface:")),
	advancedMaxHalfOpenConnectionsLabel(_("(*) Max half open connections:"))
{
	// Only show the scrollbars when they are necessary:
	set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	// Tooltips
	metadataActivateButton.set_tooltip_text(_("Lince must be reloaded to use the new value."));
	pexActivateButton.set_tooltip_text(_("Lince must be reloaded to use the new value."));
	smartBanButton.set_tooltip_text(_("Lince must be reloaded to use the new value."));
	advancedBindToInterfaceLabel.set_tooltip_text(_("Lince must be reloaded to use the new value."));
	advancedMaxHalfOpenConnectionsLabel.set_tooltip_text(_("Lince must be reloaded to use the new value."));

	// Tcp port range
	firstTcpButton.set_increments(1, 1);
	firstTcpButton.set_range(0, 65535);
	firstTcpButton.set_numeric();
	firstTcpButton.set_wrap();

	lastTcpButton.set_increments(1, 1);
	lastTcpButton.set_range(0, 65535);
	lastTcpButton.set_numeric();
	lastTcpButton.set_wrap();

	tcpRow.pack_start(tcpText1, Gtk::PACK_SHRINK , 5);
	tcpRow.pack_start(firstTcpButton, Gtk::PACK_SHRINK , 0);
	tcpRow.pack_start(tcpText2, Gtk::PACK_SHRINK , 0);
	tcpRow.pack_start(lastTcpButton, Gtk::PACK_SHRINK , 0);

	// Options VBox
	dhtFrameHBox.pack_start(dhtActivateButton,Gtk::PACK_SHRINK, 5);
	upnpFrameHBox.pack_start(upnpActivateButton,Gtk::PACK_SHRINK, 5);
	localDiscoveryFrameHBox.pack_start(localDiscoveryActivateButton,Gtk::PACK_SHRINK, 5);
	metadataFrameHBox.pack_start(metadataActivateButton, Gtk::PACK_SHRINK, 5);
	pexFrameHBox.pack_start(pexActivateButton, Gtk::PACK_SHRINK, 5);
	sbFrameHBox.pack_start(smartBanButton, Gtk::PACK_SHRINK, 5);

	optionsVBox.pack_start(dhtFrameHBox, Gtk::PACK_SHRINK, 5);
	optionsVBox.pack_start(upnpFrameHBox, Gtk::PACK_SHRINK, 5);
	optionsVBox.pack_start(localDiscoveryFrameHBox, Gtk::PACK_SHRINK, 5);
	optionsVBox.pack_start(metadataFrameHBox, Gtk::PACK_SHRINK, 5);
	optionsVBox.pack_start(pexFrameHBox, Gtk::PACK_SHRINK, 5);
	optionsVBox.pack_start(sbFrameHBox, Gtk::PACK_SHRINK, 5);

	// Advanced options Frame
	advancedMaxHalfOpenConnectionsButton.set_increments(1, 1);
	advancedMaxHalfOpenConnectionsButton.set_range(0, 65535);
	advancedMaxHalfOpenConnectionsButton.set_numeric();

	advancedFrame.set_shadow_type(Gtk::SHADOW_OUT);
	advancedFrame.set_label(_("Advanced settings"));

	advancedBindToInterfaceEntry.set_max_length(50);

	advancedFrameHBox1.pack_start(advancedBindToInterfaceLabel, Gtk::PACK_SHRINK, 5);
	advancedFrameHBox1.pack_start(advancedBindToInterfaceEntry, Gtk::PACK_SHRINK, 5);
	advancedFrameHBox2.pack_start(advancedMaxHalfOpenConnectionsLabel, Gtk::PACK_SHRINK, 5);
	advancedFrameHBox2.pack_start(advancedMaxHalfOpenConnectionsButton, Gtk::PACK_SHRINK, 5);

	advancedFrameVBox.pack_start(advancedFrameHBox1, Gtk::PACK_SHRINK, 5); advancedFrameVBox.pack_start(advancedFrameHBox2, Gtk::PACK_SHRINK, 5);

	advancedFrameHBox.pack_start(advancedFrameVBox, Gtk::PACK_EXPAND_WIDGET, 5);
	advancedFrame.add(advancedFrameHBox);
	advancedRow.pack_start(advancedFrame, Gtk::PACK_EXPAND_WIDGET, 5);

	// VBox
	vbox.pack_start(OptionsWindowsBase::headerText, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(tcpRow, Gtk::PACK_SHRINK, 5);
	vbox.pack_start(optionsVBox, Gtk::PACK_SHRINK, 5);
	vbox.pack_start(advancedRow, Gtk::PACK_SHRINK, 5);

	add(vbox);

	//Signals
	firstTcpButton.signal_value_changed().connect( sigc::mem_fun(*this, &ConnectionWindow::firstTcpButtonChanged) );
	firstTcpButton.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setFirstTcpButtonConfigValue) );

	lastTcpButton.signal_value_changed().connect( sigc::mem_fun(*this, &ConnectionWindow::lastTcpButtonChanged) );
	lastTcpButton.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setLastTcpButtonConfigValue) );

	dhtActivateButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setDhtConfigValue) );
	upnpActivateButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setUpnpConfigValue) );
	localDiscoveryActivateButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setLocalDiscoveryConfigValue) );
	metadataActivateButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setMetadataConfigValue) );
	pexActivateButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setPexConfigValue) );
	smartBanButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setSmartBanValue) );
	advancedBindToInterfaceEntry.signal_changed().connect( sigc::mem_fun(oT, &OptionsTab::setAdvancedBindToInterfaceConfigValue) );
	advancedMaxHalfOpenConnectionsButton.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setAdvancedMaxHalfOpenConnectionsConfigValue) );

}

void ConnectionWindow::firstTcpButtonChanged()
{
	if (firstTcpButton.get_value() > lastTcpButton.get_value())
	{
		lastTcpButton.set_value(firstTcpButton.get_value());
	}
}

void ConnectionWindow::lastTcpButtonChanged()
{
	if (firstTcpButton.get_value() > lastTcpButton.get_value())
	{
		lastTcpButton.set_value(firstTcpButton.get_value());
	}
}

ConnectionWindow::~ConnectionWindow()
{
}

ProxyWindow::ProxyWindow(OptionsTab *oT):
	OptionsWindowsBase("Proxy"),
	serverProxyEnableCheckButton(_("Enable connection to the server through a proxy.")),
	serverProxyNameLabel(_("Server: ")),
	serverProxyPortLabel(_("Port: ")),
	serverProxyLoginLabel(_("Username:\t")),
	serverProxyPassLabel(_("Password:\t")),
	serverProxyTypeLabel(_("Proxy type:\t")),
	clientProxyEnableCheckButton(_("Enable connection to the client through a proxy.")),
	clientProxyNameLabel(_("Server: ")),
	clientProxyPortLabel(_("Port: ")),
	clientProxyLoginLabel(_("Username:\t")),
	clientProxyPassLabel(_("Password:\t")),
	clientProxyTypeLabel(_("Proxy type:\t"))
{
	set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	// Proxy Type
	m_refTreeModel = Gtk::ListStore::create(m_Columns);
	serverProxyTypeCombo.set_model(m_refTreeModel);
	clientProxyTypeCombo.set_model(m_refTreeModel);

	Gtk::TreeModel::Row row;

	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 1;
  	row[m_Columns.m_col_name] = "Socks4";

	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 2;
  	row[m_Columns.m_col_name] = "Socks5";

	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 3;
  	row[m_Columns.m_col_name] = "Socks5 Pw";

	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 4;
  	row[m_Columns.m_col_name] = "Http";

	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 5;
  	row[m_Columns.m_col_name] = "Http Pw";


	serverProxyTypeCombo.pack_start(m_Columns.m_col_name);
	serverProxyTypeCombo.set_active(0);

	serverProxyTypeRow.pack_start(serverProxyTypeLabel, Gtk::PACK_SHRINK, 5);
	serverProxyTypeRow.pack_start(serverProxyTypeCombo, Gtk::PACK_SHRINK, 5);

	// Proxy options Frame
	serverProxyPortButton.set_increments(1, 1);
	serverProxyPortButton.set_range(0, 65535);
	serverProxyPortButton.set_numeric();
	serverProxyPortButton.set_wrap();

	serverProxyFrame.set_shadow_type(Gtk::SHADOW_OUT);
	serverProxyFrame.set_label(_("Server proxy settings"));

	serverProxyFrame2.set_shadow_type(Gtk::SHADOW_OUT);
	serverProxyFrame2.set_label(_("Authentication"));

	serverProxyPassEntry.set_visibility(false);

	// proxyFrameVBox1
	serverProxyFrameHBox1.pack_start(serverProxyEnableCheckButton, Gtk::PACK_SHRINK, 5);
	serverProxyFrameHBox2.pack_start(serverProxyNameLabel, Gtk::PACK_SHRINK, 5);
	serverProxyFrameHBox2.pack_start(serverProxyNameEntry, Gtk::PACK_SHRINK, 5);
	serverProxyFrameHBox2.pack_start(serverProxyPortLabel, Gtk::PACK_SHRINK, 5);
	serverProxyFrameHBox2.pack_start(serverProxyPortButton, Gtk::PACK_SHRINK, 5);

	serverProxyEnableCheckButton.set_active();

	// proxyFrameVBox2
	serverProxyFrameHBox3.pack_start(serverProxyLoginLabel, Gtk::PACK_SHRINK, 5);
	serverProxyFrameHBox3.pack_start(serverProxyLoginEntry, Gtk::PACK_SHRINK, 0);
	serverProxyFrameHBox4.pack_start(serverProxyPassLabel, Gtk::PACK_SHRINK, 5);
	serverProxyFrameHBox4.pack_start(serverProxyPassEntry, Gtk::PACK_SHRINK, 0);

	serverProxyFrameVBox.pack_start(serverProxyFrameHBox1, Gtk::PACK_SHRINK, 5); serverProxyFrameVBox.pack_start(serverProxyFrameHBox2, Gtk::PACK_SHRINK, 5);
	serverProxyFrameVBox.pack_start(serverProxyTypeRow, Gtk::PACK_SHRINK, 5); serverProxyFrameVBox.pack_start(serverProxyFrame2, Gtk::PACK_SHRINK, 5);

	serverProxyFrameVBox2.pack_start(serverProxyFrameHBox3, Gtk::PACK_SHRINK, 5); serverProxyFrameVBox2.pack_start(serverProxyFrameHBox4, Gtk::PACK_SHRINK, 5);

	serverProxyFrame2.add(serverProxyFrameVBox2);

	serverProxyFrameHBox.pack_start(serverProxyFrameVBox, Gtk::PACK_EXPAND_WIDGET, 5);
	serverProxyFrame.add(serverProxyFrameHBox);
	serverProxyRow.pack_start(serverProxyFrame, Gtk::PACK_EXPAND_WIDGET, 5);

	// Signals
	serverProxyEnableCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &ProxyWindow::setProxyEnableSeverSensitivity) );
	serverProxyEnableCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setServerProxyEnableCheckButtonConfigValue) );
	serverProxyNameEntry.signal_changed().connect( sigc::mem_fun(oT, &OptionsTab::setServerProxyNameConfigValue) );
	serverProxyPortButton.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setServerProxyPortConfigValue) );
	serverProxyLoginEntry.signal_changed().connect( sigc::mem_fun(oT, &OptionsTab::setServerProxyLoginConfigValue) );
	serverProxyPassEntry.signal_changed().connect( sigc::mem_fun(oT, &OptionsTab::setServerProxyPassConfigValue) );
	serverProxyTypeCombo.signal_changed().connect( sigc::mem_fun(oT, &OptionsTab::setServerProxyTypeConfigValue) );

	// Client side
	clientProxyTypeCombo.pack_start(m_Columns.m_col_name);
	clientProxyTypeCombo.set_active(0);

	clientProxyTypeRow.pack_start(clientProxyTypeLabel, Gtk::PACK_SHRINK, 5);
	clientProxyTypeRow.pack_start(clientProxyTypeCombo, Gtk::PACK_SHRINK, 5);

	// Proxy options Frame
	clientProxyPortButton.set_increments(1, 1);
	clientProxyPortButton.set_range(0, 65535);
	clientProxyPortButton.set_numeric();
	clientProxyPortButton.set_wrap();

	clientProxyFrame.set_shadow_type(Gtk::SHADOW_OUT);
	clientProxyFrame.set_label(_("Client proxy settings"));

	clientProxyFrame2.set_shadow_type(Gtk::SHADOW_OUT);
	clientProxyFrame2.set_label(_("Authentication"));

	clientProxyPassEntry.set_visibility(false);

	// proxyFrameVBox1
	clientProxyFrameHBox1.pack_start(clientProxyEnableCheckButton, Gtk::PACK_SHRINK, 5);
	clientProxyFrameHBox2.pack_start(clientProxyNameLabel, Gtk::PACK_SHRINK, 5);
	clientProxyFrameHBox2.pack_start(clientProxyNameEntry, Gtk::PACK_SHRINK, 5);
	clientProxyFrameHBox2.pack_start(clientProxyPortLabel, Gtk::PACK_SHRINK, 5);
	clientProxyFrameHBox2.pack_start(clientProxyPortButton, Gtk::PACK_SHRINK, 5);

	clientProxyEnableCheckButton.set_active();

	// proxyFrameVBox2
	clientProxyFrameHBox3.pack_start(clientProxyLoginLabel, Gtk::PACK_SHRINK, 5);
	clientProxyFrameHBox3.pack_start(clientProxyLoginEntry, Gtk::PACK_SHRINK, 0);
	clientProxyFrameHBox4.pack_start(clientProxyPassLabel, Gtk::PACK_SHRINK, 5);
	clientProxyFrameHBox4.pack_start(clientProxyPassEntry, Gtk::PACK_SHRINK, 0);

	clientProxyFrameVBox.pack_start(clientProxyFrameHBox1, Gtk::PACK_SHRINK, 5); clientProxyFrameVBox.pack_start(clientProxyFrameHBox2, Gtk::PACK_SHRINK, 5);
	clientProxyFrameVBox.pack_start(clientProxyTypeRow, Gtk::PACK_SHRINK, 5); clientProxyFrameVBox.pack_start(clientProxyFrame2, Gtk::PACK_SHRINK, 5);

	clientProxyFrameVBox2.pack_start(clientProxyFrameHBox3, Gtk::PACK_SHRINK, 5); clientProxyFrameVBox2.pack_start(clientProxyFrameHBox4, Gtk::PACK_SHRINK, 5);

	clientProxyFrame2.add(clientProxyFrameVBox2);

	clientProxyFrameHBox.pack_start(clientProxyFrameVBox, Gtk::PACK_EXPAND_WIDGET, 5);
	clientProxyFrame.add(clientProxyFrameHBox);
	clientProxyRow.pack_start(clientProxyFrame, Gtk::PACK_EXPAND_WIDGET, 5);

	// Signals
	clientProxyEnableCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &ProxyWindow::setProxyEnableClientSensitivity) );
	clientProxyEnableCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setClientProxyEnableCheckButtonConfigValue) );
	clientProxyNameEntry.signal_changed().connect( sigc::mem_fun(oT, &OptionsTab::setClientProxyNameConfigValue) );
	clientProxyPortButton.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setClientProxyPortConfigValue) );
	clientProxyLoginEntry.signal_changed().connect( sigc::mem_fun(oT, &OptionsTab::setClientProxyLoginConfigValue) );
	clientProxyPassEntry.signal_changed().connect( sigc::mem_fun(oT, &OptionsTab::setClientProxyPassConfigValue) );
	clientProxyTypeCombo.signal_changed ().connect( sigc::mem_fun(oT, &OptionsTab::setClientProxyTypeConfigValue) );


	vbox.pack_start(OptionsWindowsBase::headerText, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(serverProxyRow, Gtk::PACK_SHRINK, 5);
	vbox.pack_start(clientProxyRow, Gtk::PACK_SHRINK, 5);

	add(vbox);
}

void ProxyWindow::setProxyEnableSeverSensitivity()
{
	if (serverProxyEnableCheckButton.get_active())
	{
		serverProxyNameEntry.property_sensitive () = true;
		serverProxyPortButton.property_sensitive () = true;
		serverProxyTypeCombo.property_sensitive () = true;
		serverProxyLoginEntry.property_sensitive () = true;
		serverProxyPassEntry.property_sensitive () = true;
	}
	else
	{
		serverProxyNameEntry.property_sensitive () = false;
		serverProxyPortButton.property_sensitive () = false;
		serverProxyTypeCombo.property_sensitive () = false;
		serverProxyLoginEntry.property_sensitive () = false;
		serverProxyPassEntry.property_sensitive () = false;
		serverProxyLoginEntry.property_sensitive () = false;
		serverProxyPassEntry.property_sensitive () = false;
	}
}

void ProxyWindow::setProxyEnableClientSensitivity()
{
	if (clientProxyEnableCheckButton.get_active())
	{
		clientProxyNameEntry.property_sensitive () = true;
		clientProxyPortButton.property_sensitive () = true;
		clientProxyTypeCombo.property_sensitive () = true;
		clientProxyLoginEntry.property_sensitive () = true;
		clientProxyPassEntry.property_sensitive () = true;
	}
	else
	{
		clientProxyNameEntry.property_sensitive () = false;
		clientProxyPortButton.property_sensitive () = false;
		clientProxyTypeCombo.property_sensitive () = false;
		clientProxyLoginEntry.property_sensitive () = false;
		clientProxyPassEntry.property_sensitive () = false;
		clientProxyLoginEntry.property_sensitive () = false;
		clientProxyPassEntry.property_sensitive () = false;
	}
}

ProxyWindow::~ProxyWindow()
{
}

EncryptionWindow::EncryptionWindow(OptionsTab *oT):
	OptionsWindowsBase(_("Encryption")),
	encryptionEnableCheckButton(_("Allow encryption.")),
	incomingRadioButton1(_("Only encrypted connections are allowed.")),
	incomingRadioButton2(_("Allow encrypted and non-encrypted connections")),
	incomingRadioButton3(_("Only non-encrypted connections are allowed.")),
	outgoingRadioButton1(_("Only encrypted connections are allowed.")),
	outgoingRadioButton2(_("Allow encrypted and non-encrypted connections")),
	outgoingRadioButton3(_("Only non-encrypted connections are allowed.")),
	allowedEncryptionButton1(_("Only the handshake is encrypted.")),
	allowedEncryptionButton2(_("The entire stream is encrypted with RC4.")),
	allowedEncryptionButton3(_("RC4 and plaintext connections are allowed.")),
	preferRC4(_("Prefer the RC4 encrypted stream."))
{
	// Only show the scrollbars when they are necessary:
	set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	encryptionEnableRow.pack_start(encryptionEnableCheckButton, Gtk::PACK_SHRINK , 5);
	preferRC4Row.pack_start(preferRC4, Gtk::PACK_SHRINK , 5);

	encryptionEnableCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &EncryptionWindow::setEncryptionSensitivity) );
	encryptionEnableCheckButton.set_active();

	// Incoming Connections
	incomingFrame.set_shadow_type(Gtk::SHADOW_OUT);
	incomingFrame.set_label(_("Inconming connections"));

	incomingFrameVBox.pack_start(incomingRadioButton1, Gtk::PACK_SHRINK, 5);
	incomingFrameVBox.pack_start(incomingRadioButton2, Gtk::PACK_SHRINK, 5);
	incomingFrameVBox.pack_start(incomingRadioButton3, Gtk::PACK_SHRINK, 5);
	incomingFrame.add(incomingFrameVBox);

	incomingFrameRow.pack_start(incomingFrame, Gtk::PACK_EXPAND_WIDGET, 5);

	Gtk::RadioButtonGroup incomingRBG = incomingRadioButton1.get_group();

	incomingRadioButton2.set_group(incomingRBG);
	incomingRadioButton3.set_group(incomingRBG);

	// Outgoing Connections
	outgoingFrame.set_shadow_type(Gtk::SHADOW_OUT);
	outgoingFrame.set_label(_("Outgoing connections"));

	outgoingFrameVBox.pack_start(outgoingRadioButton1, Gtk::PACK_SHRINK, 5);
	outgoingFrameVBox.pack_start(outgoingRadioButton2, Gtk::PACK_SHRINK, 5);
	outgoingFrameVBox.pack_start(outgoingRadioButton3, Gtk::PACK_SHRINK, 5);
	outgoingFrame.add(outgoingFrameVBox);

	outgoingFrameRow.pack_start(outgoingFrame, Gtk::PACK_EXPAND_WIDGET, 5);

	Gtk::RadioButtonGroup outgoingRBG = outgoingRadioButton1.get_group();

	outgoingRadioButton2.set_group(outgoingRBG);
	outgoingRadioButton3.set_group(outgoingRBG);

	// Encryption Allowed
	allowedEncryption.set_shadow_type(Gtk::SHADOW_OUT);
	allowedEncryption.set_label(_("Allowed Encryption"));

	allowedEncryptionVBox.pack_start(allowedEncryptionButton1, Gtk::PACK_SHRINK, 5);
	allowedEncryptionVBox.pack_start(allowedEncryptionButton2, Gtk::PACK_SHRINK, 5);
	allowedEncryptionVBox.pack_start(allowedEncryptionButton3, Gtk::PACK_SHRINK, 5);
	allowedEncryption.add(allowedEncryptionVBox);

	allowedEncryptionRow.pack_start(allowedEncryption, Gtk::PACK_EXPAND_WIDGET, 5);

	Gtk::RadioButtonGroup allowedEncryptionRBG = allowedEncryptionButton1.get_group();

	allowedEncryptionButton2.set_group(allowedEncryptionRBG);
	allowedEncryptionButton3.set_group(allowedEncryptionRBG);

	// Signals
	encryptionEnableCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setEncryptionEnableButtonConfigValue) );
	preferRC4.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setPreferRC4ConfigValue) );
	incomingRadioButton1.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setInconmingConnectionsButtonConfigValue) );
	incomingRadioButton2.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setInconmingConnectionsButtonConfigValue) );
	incomingRadioButton3.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setInconmingConnectionsButtonConfigValue) );
	outgoingRadioButton1.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setOutgoingConnectionsButtonConfigValue) );
	outgoingRadioButton2.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setOutgoingConnectionsButtonConfigValue) );
	outgoingRadioButton3.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setOutgoingConnectionsButtonConfigValue) );
	allowedEncryptionButton1.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setAllowedEncryptionButtonConfigValue) );
	allowedEncryptionButton2.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setAllowedEncryptionButtonConfigValue) );
	allowedEncryptionButton3.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setAllowedEncryptionButtonConfigValue) );


	vbox.pack_start(OptionsWindowsBase::headerText, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(encryptionEnableRow, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(incomingFrameRow, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(outgoingFrameRow, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(allowedEncryptionRow, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(preferRC4Row, Gtk::PACK_SHRINK , 5);
	add(vbox);
}

int EncryptionWindow::getInconmingConnectionsButtonData()
{
	if (incomingRadioButton1.get_active())
		return 1;
	else if ((incomingRadioButton3.get_active()))
		return 3;
	else return 2;
}

void EncryptionWindow::setInconmingConnectionsButtonData(int value)
{
	if (value == 1) incomingRadioButton1.set_active();
	else if (value == 3) incomingRadioButton3.set_active();
	else incomingRadioButton2.set_active();
}

int EncryptionWindow::getOutgoingConnectionsButtonData()
{
	if (outgoingRadioButton1.get_active())
		return 1;
	else if ((outgoingRadioButton3.get_active()))
		return 3;
	else return 2;
}

void EncryptionWindow::setOutgoingConnectionsButtonData(int value)
{
	if (value == 1) outgoingRadioButton1.set_active();
	else if (value == 3) outgoingRadioButton3.set_active();
	else outgoingRadioButton2.set_active();
}

int EncryptionWindow::getAllowedEncryptionButtonData()
{
	if (allowedEncryptionButton1.get_active())
		return 1;
	else if ((allowedEncryptionButton2.get_active()))
		return 2;
	else return 3;
}

void EncryptionWindow::setAllowedEncryptionButtonData(int value)
{
	if (value == 1) allowedEncryptionButton1.set_active();
	else if (value == 2) allowedEncryptionButton2.set_active();
	else allowedEncryptionButton3.set_active();
}

void EncryptionWindow::setEncryptionSensitivity()
{
	if (encryptionEnableCheckButton.get_active())
	{
		incomingRadioButton1.property_sensitive () = true;
		incomingRadioButton2.property_sensitive () = true;
		incomingRadioButton3.property_sensitive () = true;
		outgoingRadioButton1.property_sensitive () = true;
		outgoingRadioButton2.property_sensitive () = true;
		outgoingRadioButton3.property_sensitive () = true;
		allowedEncryptionButton1.property_sensitive () = true;
		allowedEncryptionButton2.property_sensitive () = true;
		allowedEncryptionButton3.property_sensitive () = true;
		preferRC4Row.property_sensitive () = true;
	}
	else
	{
		incomingRadioButton1.property_sensitive () = false;
		incomingRadioButton2.property_sensitive () = false;
		incomingRadioButton3.property_sensitive () = false;
		outgoingRadioButton1.property_sensitive () = false;
		outgoingRadioButton2.property_sensitive () = false;
		outgoingRadioButton3.property_sensitive () = false;
		allowedEncryptionButton1.property_sensitive () = false;
		allowedEncryptionButton2.property_sensitive () = false;
		allowedEncryptionButton3.property_sensitive () = false;
		preferRC4Row.property_sensitive () = false;
	}
}

EncryptionWindow::~EncryptionWindow()
{
}

TransferenceWindow::TransferenceWindow(OptionsTab * oT):
	OptionsWindowsBase(_("Transference")),
	maxUploadLabel(_("Maximum upload speed (in KB/s)")),
	maxUploadCheckButton(_("Disable")),
	maxDownloadLabel(_("Maximum download speed (in KB/s)")),
	maxDownloadCheckButton(_("Disable")),
	maxConnectsLabel(_("Maximum number of connections:")),
	maxConnectsCheckButton(_("Disable")),
	maxUploadSlotsLabel(_("Maximum number of upload slots (global):")),
	maxUploadSlotsCheckButton(_("Disable")),
	shareRatioLabel1(_("Share ratio:\t")),
	shareRatioLabel2("1 KB DL ="),
	shareRatioLabel3("KB UP"),
	shareRatioCheckButton(_("Disable")),
	allowConnectionsFromTheSameIp(_("Allow multiple connections from the same IP.")),
	prioritizeFirstPartOfFiles(_("Give priority to the first part of files."))
{
	// Only show the scrollbars when they are necessary:
	set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	Gtk::Table *table = Gtk::manage(new Gtk::Table(4, 3, false));

	maxUploadButton.set_increments(1, 5);
	maxUploadButton.set_range(1, 20000);
	maxUploadButton.set_numeric();

	maxDownloadButton.set_increments(1, 5);
	maxDownloadButton.set_range(1, 20000);
	maxDownloadButton.set_numeric();

	maxConnectsButton.set_increments(1, 5);
	maxConnectsButton.set_range(1, 10000);
	maxConnectsButton.set_numeric();

	maxUploadSlotsButton.set_increments(1, 5);
	maxUploadSlotsButton.set_range(1, 10000);
	maxUploadSlotsButton.set_numeric();

	shareRatioButton.set_increments(0.1, 1);
	shareRatioButton.set_range(1, 1000);
	shareRatioButton.set_digits(1);
	shareRatioButton.set_numeric();

	// Max Upload
	maxUploadCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &TransferenceWindow::setMaxUploadButtonSensitivity) );
	maxUploadCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setMaxUploadCheckButtonConfigValue) );
	maxUploadButton.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setMaxUploadButtonConfigValue) );

	maxUploadLabel.property_xalign()  = 0;
	table->attach(maxUploadLabel, 0, 1, 0, 1, Gtk::FILL, Gtk::SHRINK, 5, 5);
	table->attach(maxUploadButton, 1, 2, 0, 1, Gtk::SHRINK, Gtk::SHRINK, 10, 5);
	table->attach(maxUploadCheckButton, 2, 3, 0, 1, Gtk::SHRINK, Gtk::SHRINK, 0, 5);

	// Max Download
	maxDownloadCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &TransferenceWindow::setMaxDownloadButtonSensitivity) );
	maxDownloadCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setMaxDownloadCheckButtonConfigValue) );
	maxDownloadButton.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setMaxDownloadButtonConfigValue) );

	maxDownloadLabel.property_xalign()  = 0;
	table->attach(maxDownloadLabel, 0, 1, 1, 2, Gtk::FILL, Gtk::SHRINK, 5, 5);
	table->attach(maxDownloadButton, 1, 2, 1, 2, Gtk::SHRINK, Gtk::SHRINK, 10, 5);
	table->attach(maxDownloadCheckButton, 2, 3, 1, 2, Gtk::SHRINK, Gtk::SHRINK, 0, 5);


	// Max Connects
	maxConnectsCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &TransferenceWindow::setMaxConnectsButtonSensitivity) );
	maxConnectsCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setMaxConnectsCheckButtonConfigValue) );
	maxConnectsButton.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setMaxConnectsButtonConfigValue) );

	maxConnectsLabel.property_xalign()  = 0;
	table->attach(maxConnectsLabel, 0, 1, 2, 3, Gtk::FILL, Gtk::SHRINK, 5, 5);
	table->attach(maxConnectsButton, 1, 2, 2, 3, Gtk::SHRINK, Gtk::SHRINK, 10, 5);
	table->attach(maxConnectsCheckButton, 2, 3, 2, 3, Gtk::SHRINK, Gtk::SHRINK, 0, 5);

	// Max Upload Slots
	maxUploadSlotsCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &TransferenceWindow::setMaxUploadSlotsButtonSensitivity) );
	maxUploadSlotsCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setMaxUploadSlotsCheckButtonConfigValue) );
	maxUploadSlotsButton.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setMaxUploadSlotsButtonConfigValue) );

	maxUploadSlotsLabel.property_xalign()  = 0;
	table->attach(maxUploadSlotsLabel, 0, 1, 3, 4, Gtk::FILL, Gtk::SHRINK, 5, 5);
	table->attach(maxUploadSlotsButton, 1, 2, 3, 4, Gtk::SHRINK, Gtk::SHRINK, 10, 5);
	table->attach(maxUploadSlotsCheckButton, 2, 3, 3, 4, Gtk::SHRINK, Gtk::SHRINK, 0, 5);

	//Share Ratio
	shareRatioCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &TransferenceWindow::setShareRatioButtonSensitivity) );
	shareRatioCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setShareRatioCheckButtonConfigValue) );
	shareRatioButton.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setShareRatioConfigValue) );

	Gtk::HBox *shareRatioHBox = Gtk::manage(new Gtk::HBox());

	shareRatioHBox->pack_start(shareRatioLabel1, Gtk::PACK_SHRINK , 5);
	shareRatioHBox->pack_start(shareRatioLabel2, Gtk::PACK_SHRINK , 5);
	shareRatioHBox->pack_start(shareRatioButton, Gtk::PACK_SHRINK , 5);
	shareRatioHBox->pack_start(shareRatioLabel3, Gtk::PACK_SHRINK , 5);
	shareRatioHBox->pack_start(shareRatioCheckButton, Gtk::PACK_SHRINK , 5);

	// Connections from the same Ip
	allowConnectionsFromTheSameIp.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setAllowConnectionsFromTheSameIpConfigValue) );
	Gtk::HBox *allowConnectionsFromTheSameIpRow = Gtk::manage(new Gtk::HBox());
	allowConnectionsFromTheSameIpRow->pack_start(allowConnectionsFromTheSameIp, Gtk::PACK_SHRINK , 5);

	// Prioritize the first part of the files.
	prioritizeFirstPartOfFiles.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setPrioritizeFirstPartOfFilesConfigValue) );
	Gtk::HBox *prioritizeFirstPartOfFilesHBox = Gtk::manage(new Gtk::HBox());
	prioritizeFirstPartOfFilesHBox->pack_start(prioritizeFirstPartOfFiles, Gtk::PACK_SHRINK , 5);


	vbox.pack_start(OptionsWindowsBase::headerText, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(*table, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(*shareRatioHBox, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(*allowConnectionsFromTheSameIpRow, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(*prioritizeFirstPartOfFilesHBox, Gtk::PACK_SHRINK , 5);

	add(vbox);
}

int TransferenceWindow::getShareRatioButton1()
{
	if (shareRatioButton.get_value() - shareRatioButton.get_value_as_int() >= 0)
		return shareRatioButton.get_value_as_int();
	else
		return shareRatioButton.get_value_as_int() - 1;
}

int TransferenceWindow::getShareRatioButton2() {
	if (shareRatioButton.get_value() - shareRatioButton.get_value_as_int() >= 0)
		return (int)((shareRatioButton.get_value() - shareRatioButton.get_value_as_int()) * 10);
	else
		return (int)((shareRatioButton.get_value() - (shareRatioButton.get_value_as_int()-1)) * 10);
}

void TransferenceWindow::setMaxUploadButtonSensitivity()
{
	if (maxUploadCheckButton.get_active())
		maxUploadButton.property_sensitive () = false;
	else
		maxUploadButton.property_sensitive () = true;
}

void TransferenceWindow::setMaxDownloadButtonSensitivity()
{
	if (maxDownloadCheckButton.get_active())
		maxDownloadButton.property_sensitive () = false;
	else
		maxDownloadButton.property_sensitive () = true;
}

void TransferenceWindow::setMaxConnectsButtonSensitivity()
{
	if (maxConnectsCheckButton.get_active())
		maxConnectsButton.property_sensitive () = false;
	else
		maxConnectsButton.property_sensitive () = true;
}

void TransferenceWindow::setMaxUploadSlotsButtonSensitivity()
{
	if (maxUploadSlotsCheckButton.get_active())
		maxUploadSlotsButton.property_sensitive () = false;
	else
		maxUploadSlotsButton.property_sensitive () = true;
}

void TransferenceWindow::setShareRatioButtonSensitivity()
{
	if (shareRatioCheckButton.get_active())
	{
		shareRatioButton.property_sensitive () = false;
		shareRatioLabel2.property_sensitive () = false;
		shareRatioLabel3.property_sensitive () = false;
	}
	else
	{
		shareRatioButton.property_sensitive () = true;
		shareRatioLabel2.property_sensitive () = true;
		shareRatioLabel3.property_sensitive () = true;
	}
}

TransferenceWindow::~TransferenceWindow()
{
}

FilesWindow::FilesWindow(OptionsTab *oT):
	OptionsWindowsBase(_("Files")),
	directoryLabel(_("Default directory")),
	downloadDefaultDirectoryCheckButton(_("Download in the default directory.")),
	storageTypeLabel(_("Storage type: ")),
	//fastResumeCheckButton(_("Use fast resume mode")),
	//fastResumeTimeUpdateLabel(_("Update resume data every:")),
	//minutesLabel(_("minutes")),
	confirmDataEliminationCheckButton(_("Confirm data elimination")),
	askWhenLoadingTorrentsUsingDbusRequestsCheckButton(_("Open store dialog when loading torrents from other applications"))
{
	// Only show the scrollbars when they are necessary:
	set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	// Directory Frame
	directoryFrame.set_shadow_type(Gtk::SHADOW_OUT);
	directoryFrame.set_label(_("Default directory's options"));

	directorySelectButtonImage.set(Gtk::Stock::DIRECTORY, Gtk::IconSize(1));
	directorySelectButton.set_image(directorySelectButtonImage);

	directoryFrameHBox2.pack_start(downloadDefaultDirectoryCheckButton, Gtk::PACK_SHRINK , 5);

	directoryFrameHBox.pack_start(directoryLabel, Gtk::PACK_SHRINK , 10);
	directoryFrameHBox.pack_start(directoryEntry, Gtk::PACK_EXPAND_WIDGET , 0);
	directoryFrameHBox.pack_start(directorySelectButton, Gtk::PACK_SHRINK , 5);

	directoryFrameVBox.pack_start(directoryFrameHBox, Gtk::PACK_SHRINK , 5);
	directoryFrameVBox.pack_start(directoryFrameHBox2, Gtk::PACK_SHRINK , 5);

	directoryFrame.add(directoryFrameVBox);
	directoryRow.pack_start(directoryFrame, Gtk::PACK_EXPAND_WIDGET , 5);

	// Storage allocation
	m_refTreeModel = Gtk::ListStore::create(m_Columns);
	storageTypeCombo.set_model(m_refTreeModel);

	Gtk::TreeModel::Row row;

	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 1;
  	row[m_Columns.m_col_name] = "Full Allocation";

	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 2;
  	row[m_Columns.m_col_name] = "Compact";

	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 3;
  	row[m_Columns.m_col_name] = "Sparse";

	storageTypeCombo.pack_start(m_Columns.m_col_name);
	storageTypeCombo.set_active(2);

	storageTypeRow.pack_start(storageTypeLabel, Gtk::PACK_SHRINK, 5);
	storageTypeRow.pack_start(storageTypeCombo, Gtk::PACK_SHRINK, 5);

	// Fast Resume
	/*fastResumeCheckButton.set_active();

	fastResumeRow.pack_start(fastResumeCheckButton, Gtk::PACK_SHRINK , 5);
	fastResumeTimeUpdateButton.set_increments(1, 1);
	fastResumeTimeUpdateButton.set_range(1, 60);
	fastResumeTimeUpdateButton.set_numeric();
	fastResumeTimeUpdateRow.pack_start(fastResumeTimeUpdateLabel, Gtk::PACK_SHRINK , 40);
	fastResumeTimeUpdateRow.pack_start(fastResumeTimeUpdateButton, Gtk::PACK_SHRINK , 0);
	fastResumeTimeUpdateRow.pack_start(minutesLabel, Gtk::PACK_SHRINK , 10);*/

	// Data elimination
	confirmDataEliminationRow.pack_start(confirmDataEliminationCheckButton, Gtk::PACK_SHRINK , 5);

	// Ask when loading torrents from dbus requests
	askWhenLoadingTorrentsUsingDbusRequestsRow.pack_start(askWhenLoadingTorrentsUsingDbusRequestsCheckButton, Gtk::PACK_SHRINK , 5);

	// Signals
	directorySelectButton.signal_clicked().connect( sigc::mem_fun(*this, &FilesWindow::selectAndSetDirectory) );
	directoryEntry.signal_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDirectoryEntryConfigValue) );
	downloadDefaultDirectoryCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadDefaultDirectoryCheckButtonConfigValue) );
	storageTypeCombo.signal_changed().connect( sigc::mem_fun(oT, &OptionsTab::setStorageTypeConfigValue) );
	//fastResumeTimeUpdateButton.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setFastResumeTimeUpdateButtonConfigValue) );
	//fastResumeCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &FilesWindow::setFastResumeTimeUpdateButtonSensitivity) );
	//fastResumeCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setFastResumeCheckButtonConfigValue) );
	confirmDataEliminationCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setConfirmDataEliminationCheckButtonConfigValue) );
	askWhenLoadingTorrentsUsingDbusRequestsCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setAskWhenLoadingTorrentsUsingDbusRequestsConfigValue) );


	vbox.pack_start(OptionsWindowsBase::headerText, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(directoryRow, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(storageTypeRow, Gtk::PACK_SHRINK , 5);
	//vbox.pack_start(fastResumeRow, Gtk::PACK_SHRINK , 5);
	//vbox.pack_start(fastResumeTimeUpdateRow, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(confirmDataEliminationRow, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(askWhenLoadingTorrentsUsingDbusRequestsRow, Gtk::PACK_SHRINK , 5);
	add(vbox);
}

void FilesWindow::selectAndSetDirectory()
{
	Glib::ustring directory;

	directory = Utils::getDirectoryFromDialog(_("Select default save path for downloads"));

	if (directory != "") directoryEntry.set_text(directory);
}

void FilesWindow::setFastResumeTimeUpdateButtonSensitivity()
{
	/*if (fastResumeCheckButton.get_active())
		fastResumeTimeUpdateButton.property_sensitive () = true;
	else
		fastResumeTimeUpdateButton.property_sensitive () = false;
	*/
}

FilesWindow::~FilesWindow()
{
}

TorrentsOptionsWindow::TorrentsOptionsWindow(OptionsTab *oT):
	OptionsWindowsBase("Torrents"),
	saveTorrentsCheckButton(_("Save .torrent files")),
	saveDirectoryLabel(_("Directory:")),
	loadTorrentsCheckButton(_("Load .torrent files")),
	loadDirectoryLabel(_("Directory:")),
	allowChangeDirectory(_("Allow to change the directory during the session")),
	removeOriginalTorrent(_("Remove original torrent file")),
	removeTorrentWhenFinished(_("Erase torrent file when the torrent is removed"))
{
	// Only show the scrollbars when they are necessary:
	set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	// Load .torrents files
	Gtk::HBox *loadTorrentsRow, *loadDirectoryRow, *allowChangeDirectoryRow;
	loadTorrentsRow = Gtk::manage(new Gtk::HBox());
	loadDirectoryRow = Gtk::manage(new Gtk::HBox());
	allowChangeDirectoryRow = Gtk::manage(new Gtk::HBox());

	loadTorrentsCheckButton.set_active();

	loadDirectorySelectButtonImage.set(Gtk::Stock::DIRECTORY, Gtk::IconSize(1));
	loadDirectorySelectButton.set_image(loadDirectorySelectButtonImage);

	loadTorrentsRow->pack_start(loadTorrentsCheckButton, Gtk::PACK_SHRINK , 5);
	loadDirectoryRow->pack_start(loadDirectoryLabel, Gtk::PACK_SHRINK , 25);
	loadDirectoryRow->pack_start(loadDirectoryEntry, Gtk::PACK_EXPAND_WIDGET , 0);
	loadDirectoryRow->pack_start(loadDirectorySelectButton, Gtk::PACK_SHRINK , 5);
	allowChangeDirectoryRow->pack_start(allowChangeDirectory, Gtk::PACK_SHRINK , 25);

	// Save torrents file
	Gtk::HBox *saveTorrentsRow, *saveDirectoryRow, *removeOriginalTorrentRow, *removeTorrentWhenFinishedRow;
	saveTorrentsRow = Gtk::manage(new Gtk::HBox());
	saveDirectoryRow = Gtk::manage(new Gtk::HBox());
	removeOriginalTorrentRow = Gtk::manage(new Gtk::HBox());
	removeTorrentWhenFinishedRow = Gtk::manage(new Gtk::HBox());

	saveTorrentsCheckButton.set_active();

	saveDirectorySelectButtonImage.set(Gtk::Stock::DIRECTORY, Gtk::IconSize(1));
	saveDirectorySelectButton.set_image(saveDirectorySelectButtonImage);

	saveTorrentsRow->pack_start(saveTorrentsCheckButton, Gtk::PACK_SHRINK , 5);
	saveDirectoryRow->pack_start(saveDirectoryLabel, Gtk::PACK_SHRINK , 25);
	saveDirectoryRow->pack_start(saveDirectoryEntry, Gtk::PACK_EXPAND_WIDGET , 0);
	saveDirectoryRow->pack_start(saveDirectorySelectButton, Gtk::PACK_SHRINK , 5);
	removeOriginalTorrentRow->pack_start(removeOriginalTorrent, Gtk::PACK_SHRINK , 25);
	removeTorrentWhenFinishedRow->pack_start(removeTorrentWhenFinished, Gtk::PACK_SHRINK , 25);

	// Signals
	saveTorrentsCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &TorrentsOptionsWindow::setSaveDirectoryRowSensitivity) );
	saveDirectorySelectButton.signal_clicked().connect( sigc::mem_fun(*this, &TorrentsOptionsWindow::selectAndSetSaveDirectory) );
	loadTorrentsCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &TorrentsOptionsWindow::setLoadDirectoryRowSensitivity) );
	loadDirectorySelectButton.signal_clicked().connect( sigc::mem_fun(*this, &TorrentsOptionsWindow::selectAndSetLoadDirectory) );

	saveTorrentsCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setSaveTorrentsCheckButtonConfigValue) );
	saveDirectoryEntry.signal_changed().connect( sigc::mem_fun(oT, &OptionsTab::setSaveDirectoryEntryConfigValue) );
	removeOriginalTorrent.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setRemoveOriginalTorrentConfigValue) );
	removeTorrentWhenFinished.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setRemoveTorrentWhenFinishedConfigValue) );
	loadTorrentsCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setLoadTorrentsCheckButtonConfigValue) );
	loadDirectoryEntry.signal_changed().connect( sigc::mem_fun(oT, &OptionsTab::setLoadDirectoryEntryConfigValue) );
	allowChangeDirectory.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setAllowChangeDirectoryConfigValue) );


	vbox.pack_start(OptionsWindowsBase::headerText, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(*loadTorrentsRow, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(*loadDirectoryRow, Gtk::PACK_SHRINK, 5);
	vbox.pack_start(*allowChangeDirectoryRow, Gtk::PACK_SHRINK, 5);
	vbox.pack_start(*saveTorrentsRow, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(*saveDirectoryRow, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(*removeOriginalTorrentRow, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(*removeTorrentWhenFinishedRow, Gtk::PACK_SHRINK , 5);
	add(vbox);
}

void TorrentsOptionsWindow::selectAndSetSaveDirectory()
{
	Glib::ustring directory;

	directory = Utils::getDirectoryFromDialog(_("Select save path for torrents"));

	if (directory != "") saveDirectoryEntry.set_text(directory);
}

void TorrentsOptionsWindow::setSaveDirectoryRowSensitivity()
{
	if (saveTorrentsCheckButton.get_active())
	{
		saveDirectoryLabel.property_sensitive () = true;
		saveDirectoryEntry.property_sensitive () = true;
		saveDirectorySelectButton.property_sensitive () = true;
		removeOriginalTorrent.property_sensitive () = true;
		removeTorrentWhenFinished.property_sensitive () = true;
	}
	else
	{
		saveDirectoryLabel.property_sensitive () = false;
		saveDirectoryEntry.property_sensitive () = false;
		saveDirectorySelectButton.property_sensitive () = false;
		removeOriginalTorrent.property_sensitive () = false;
		removeTorrentWhenFinished.property_sensitive () = false;
	}
}

void TorrentsOptionsWindow::selectAndSetLoadDirectory()
{
	Glib::ustring directory;

	directory = Utils::getDirectoryFromDialog(_("Select load path for torrents"));

	if (directory != "") loadDirectoryEntry.set_text(directory);
}

void TorrentsOptionsWindow::setLoadDirectoryRowSensitivity()
{
	if (loadTorrentsCheckButton.get_active())
	{
		loadDirectoryLabel.property_sensitive () = true;
		loadDirectoryEntry.property_sensitive () = true;
		loadDirectorySelectButton.property_sensitive () = true;
		allowChangeDirectory.property_sensitive () = true;
	}
	else
	{
		loadDirectoryLabel.property_sensitive () = false;
		loadDirectoryEntry.property_sensitive () = false;
		loadDirectorySelectButton.property_sensitive () = false;
		allowChangeDirectory.property_sensitive () = false;
	}
}

TorrentsOptionsWindow::~TorrentsOptionsWindow()
{
}

StatisticsWindow::StatisticsWindow(OptionsTab * oT):
	OptionsWindowsBase(_("Statistics")),
	useStatisticsTabCheckButton(_("Show graphic statistics")),
	updateTimeLabel(_("Update statistics tab every: ")),
	secondsLabel(_("seconds"))
{
	// Only show the scrollbars when they are necessary:
	set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	// Statistics Tab
	statisticsTabFrame.set_shadow_type(Gtk::SHADOW_OUT);
	statisticsTabFrame.set_label(_("Statistics tab"));
	useStatisticsTabCheckButton.set_active();
	useStatisticsTabCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &StatisticsWindow::setStatisticsTabSensivity) );

	updateTimeButton.set_increments(1, 1);
	updateTimeButton.set_range(1, 15);
	updateTimeButton.set_numeric();

	updateTimeRow.pack_start(updateTimeLabel, Gtk::PACK_SHRINK, 5);
	updateTimeRow.pack_start(updateTimeButton, Gtk::PACK_SHRINK , 0);
	updateTimeRow.pack_start(secondsLabel, Gtk::PACK_SHRINK , 10);

	useStatisticsTabRow.pack_start(useStatisticsTabCheckButton, Gtk::PACK_SHRINK, 5);

	statisticsTabVBox.pack_start(useStatisticsTabRow, Gtk::PACK_SHRINK, 5);
	statisticsTabVBox.pack_start(updateTimeRow, Gtk::PACK_SHRINK, 5);

	statisticsTabFrame.add(statisticsTabVBox);
	statisticsTabRow.pack_start(statisticsTabFrame, Gtk::PACK_EXPAND_WIDGET , 5);

	// Signals
	useStatisticsTabCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setStatisticsTabCheckButtonConfigValue) );
	updateTimeButton.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUpdateTimeButtonConfigValue) );

	vbox.pack_start(OptionsWindowsBase::headerText, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(statisticsTabRow, Gtk::PACK_SHRINK, 5);

	add(vbox);
}

void StatisticsWindow::setStatisticsTabSensivity()
{
	if ( useStatisticsTabCheckButton.get_active() )
	{
		updateTimeButton.property_sensitive () = true;
	}
	else
	{
		updateTimeButton.property_sensitive () = false;
	}
}

StatisticsWindow::~StatisticsWindow()
{
}

LogWindow::LogWindow(OptionsTab * oT):
	OptionsWindowsBase(_("Log")),
	activateLogCheckButton(_("Activate log tab")),
	coloredOutputCheckButton(_("Enable")),
	coloredOutputLabel(_("Use colored output:")),
	saveLogFileLabel(_("Save the log to a file:")),
	saveLogFileCheckButton(_("Enable")),
	bufferSizeLabel(_("Maximum buffer log size:")),
	kB("KB"),
	logLevelLabel(_("Log level:")),
	autoScrollLabel(_("Automatic scroll:")),
	autoScrollCheckButton(_("Enable"))
{
	// Only show the scrollbars when they are necessary:
	set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	Gtk::Table *table = Gtk::manage(new Gtk::Table(3, 2, false));
	Gtk::VBox *activateLogVbox = Gtk::manage(new Gtk::VBox);
	Gtk::HBox *activateLogRow, *activateLogBox, *bufferSizeRow, *logLevelRow;
	activateLogRow = Gtk::manage(new Gtk::HBox);
	activateLogBox = Gtk::manage(new Gtk::HBox);
	bufferSizeRow = Gtk::manage(new Gtk::HBox);
	logLevelRow = Gtk::manage(new Gtk::HBox);

	activateLogCheckButton.set_active();
	activateLogCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &LogWindow::setTabFrameSensivity) );

	bufferSizeButton.set_increments(1, 10);
	bufferSizeButton.set_range(512, 3072);
	bufferSizeButton.set_numeric();

	// Log level
	m_refTreeModel = Gtk::ListStore::create(m_Columns);
	logLevelCombo.set_model(m_refTreeModel);

	Gtk::TreeModel::Row row;

	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 1;
  	row[m_Columns.m_col_name] = _("Fatal");

	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 2;
  	row[m_Columns.m_col_name] = _("Critical");

	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 3;
  	row[m_Columns.m_col_name] = _("Warning");

	row = *(m_refTreeModel->append());
	row[m_Columns.m_col_id] = 4;
  	row[m_Columns.m_col_name] = _("Info");

	logLevelCombo.pack_start(m_Columns.m_col_name);
	logLevelCombo.set_active(4);

	activateLogBox->pack_start(activateLogCheckButton, Gtk::PACK_SHRINK, 5);

	coloredOutputLabel.property_xalign()  = 0.05;
	table->attach(coloredOutputLabel, 0, 1, 0, 1, Gtk::FILL, Gtk::SHRINK, 5, 5);
	table->attach(coloredOutputCheckButton, 1, 2, 0, 1, Gtk::SHRINK, Gtk::SHRINK, 0, 5);

	saveLogFileLabel.property_xalign()  = 0.05;
	table->attach(saveLogFileLabel, 0, 1, 1, 2, Gtk::FILL, Gtk::SHRINK, 5, 5);
	table->attach(saveLogFileCheckButton, 1, 2, 1, 2, Gtk::SHRINK, Gtk::SHRINK, 0, 5);

	autoScrollLabel.property_xalign()  = 0.05;
	table->attach(autoScrollLabel, 0, 1, 2, 3, Gtk::FILL, Gtk::SHRINK, 5, 5);
	table->attach(autoScrollCheckButton, 1, 2, 2, 3, Gtk::SHRINK, Gtk::SHRINK, 0, 5);


	bufferSizeRow->pack_start(bufferSizeLabel, Gtk::PACK_SHRINK, 5);
	bufferSizeRow->pack_start(bufferSizeButton, Gtk::PACK_SHRINK, 5);
	bufferSizeRow->pack_start(kB, Gtk::PACK_SHRINK, 5);

	logLevelRow->pack_start(logLevelLabel, Gtk::PACK_SHRINK, 5);
	logLevelRow->pack_start(logLevelCombo, Gtk::PACK_SHRINK, 5);

	activateLogVbox->pack_start(*activateLogBox, Gtk::PACK_SHRINK , 5);
	activateLogVbox->pack_start(*logLevelRow, Gtk::PACK_SHRINK, 5);
	activateLogVbox->pack_start(*table, Gtk::PACK_SHRINK, 0);
	activateLogVbox->pack_start(*bufferSizeRow, Gtk::PACK_SHRINK, 5);

	// Activate log tab frame
	Gtk::Frame *activateLogFrame = Gtk::manage(new Gtk::Frame());

	activateLogFrame->set_label(_("Log tab"));
	activateLogFrame->add(*activateLogVbox);
	activateLogFrame->set_shadow_type(Gtk::SHADOW_OUT);
	activateLogRow->pack_start(*activateLogFrame, Gtk::PACK_EXPAND_WIDGET, 5);

	vbox.pack_start(OptionsWindowsBase::headerText, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(*activateLogRow, Gtk::PACK_SHRINK , 5);

	// Signals
	activateLogCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setActivateLogCheckButtonConfigValue) );
	logLevelCombo.signal_changed().connect( sigc::mem_fun(oT, &OptionsTab::setLogLevelConfigValue) );
	coloredOutputCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setColoredOutputCheckButtonConfigValue) );
	saveLogFileCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setSaveLogFileCheckButtonConfigValue) );
	autoScrollCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setAutoScrollCheckButtonConfigValue) );
	bufferSizeButton.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setBufferSizeButtonConfigValue) );

	add(vbox);
}

void LogWindow::setTabFrameSensivity()
{
	if ( activateLogCheckButton.get_active() )
	{
		bufferSizeButton.property_sensitive () = true;
		coloredOutputCheckButton.property_sensitive () = true;
		logLevelCombo.property_sensitive () = true;
		saveLogFileCheckButton.property_sensitive () = true;
		autoScrollCheckButton.property_sensitive () = true;
	}
	else
	{
		bufferSizeButton.property_sensitive () = false;
		coloredOutputCheckButton.property_sensitive () = false;
		logLevelCombo.property_sensitive () = false;
		saveLogFileCheckButton.property_sensitive () = false;
		autoScrollCheckButton.property_sensitive () = false;
	}
}

LogWindow::~LogWindow()
{
}

NotificationsWindow::NotificationsWindow(OptionsTab * oT):
	OptionsWindowsBase(_("Notifications")),
	activateNotificationsCheckButton(_("Activate notifications")),
	notifyFinishedTorrentsCheckButton(_("Finished torrents")),
	notifyStoppedTorrentsCheckButton(_("Stopped torrents")),
	notifyStartedTorrentsCheckButton(_("Started torrents")),
	notifyQueuedTorrentsCheckButton(_("Queued torrents")),
	notifyUnqueuedTorrentsCheckButton(_("Unqueued torrents")),
	notifyRemovedTorrentsCheckButton(_("Removed torrents")),
	notifyAddedTorrentsCheckButton(_("Added torrents")),
	notifyShowFromSystrayCheckButton(_("Show from systray"))
{
	// Only show the scrollbars when they are necessary:
	set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	Gtk::Table *table = Gtk::manage(new Gtk::Table(4, 2, true));
	Gtk::Frame *notificationsFrame2;
	Gtk::VBox *notificationsVBox;
	Gtk::HBox *enableRow, *notificationsRow, *tableRow;

	notificationsFrame2 = Gtk::manage(new Gtk::Frame());
	notificationsRow = Gtk::manage(new Gtk::HBox());
	enableRow = Gtk::manage(new Gtk::HBox());
	tableRow = Gtk::manage(new Gtk::HBox());
	notificationsVBox = Gtk::manage(new Gtk::VBox());

	notificationsFrame1.set_shadow_type(Gtk::SHADOW_OUT);
	notificationsFrame2->set_shadow_type(Gtk::SHADOW_OUT);

	activateNotificationsCheckButton.set_active();
	activateNotificationsCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &NotificationsWindow::setNotificationsFrameSensivity) );

	// Fill table
	table->attach(notifyAddedTorrentsCheckButton, 0, 1, 0, 1, Gtk::FILL, Gtk::SHRINK, 5, 5);
	table->attach(notifyRemovedTorrentsCheckButton, 1, 2, 0, 1, Gtk::FILL, Gtk::SHRINK, 5, 5);
	table->attach(notifyStartedTorrentsCheckButton, 0, 1, 1, 2, Gtk::FILL, Gtk::SHRINK, 5, 5);
	table->attach(notifyStoppedTorrentsCheckButton, 1, 2, 1, 2, Gtk::FILL, Gtk::SHRINK, 5, 5);
	table->attach(notifyQueuedTorrentsCheckButton, 0, 1, 2, 3, Gtk::FILL, Gtk::SHRINK, 5, 5);
	table->attach(notifyUnqueuedTorrentsCheckButton, 1, 2, 2, 3, Gtk::FILL, Gtk::SHRINK, 5, 5);
	table->attach(notifyFinishedTorrentsCheckButton, 0, 1, 3, 4, Gtk::FILL, Gtk::SHRINK, 5, 5);

	notificationsFrame2->set_label(_("Notifications"));
	notificationsFrame2->add(*table);

	enableRow->pack_start(activateNotificationsCheckButton, Gtk::PACK_SHRINK, 5);
	tableRow->pack_start(*notificationsFrame2, Gtk::PACK_SHRINK, 5);

	notificationsVBox->pack_start(notifyShowFromSystrayCheckButton, Gtk::PACK_SHRINK, 5);
	notificationsVBox->pack_start(*tableRow, Gtk::PACK_SHRINK, 5);
	notificationsFrame1.add(*notificationsVBox);
	notificationsRow->pack_start(notificationsFrame1, Gtk::PACK_EXPAND_WIDGET, 5);

	// Vbox
	vbox.pack_start(OptionsWindowsBase::headerText, Gtk::PACK_SHRINK, 5);
	vbox.pack_start(*enableRow, Gtk::PACK_SHRINK, 5);
	vbox.pack_start(*notificationsRow, Gtk::PACK_SHRINK, 5);

	// Signals
	activateNotificationsCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setActivateNotificationsConfigValue) );
	notifyAddedTorrentsCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setNotifyAddedTorrentsConfigValue) );
	notifyRemovedTorrentsCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setNotifyRemovedTorrentsConfigValue) );
	notifyStartedTorrentsCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setNotifyStartedTorrentsConfigValue) );
	notifyStoppedTorrentsCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setNotifyStoppedTorrentsConfigValue) );
	notifyQueuedTorrentsCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setNotifyQueuedTorrentsConfigValue) );
	notifyUnqueuedTorrentsCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setNotifyUnqueuedTorrentsConfigValue) );
	notifyFinishedTorrentsCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setNotifyFinishedTorrentsConfigValue) );
	notifyShowFromSystrayCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setNotifyShowFromSystrayConfigValue) );

	add(vbox);
}

void NotificationsWindow::setNotificationsFrameSensivity()
{
	if (activateNotificationsCheckButton.get_active())
		notificationsFrame1.property_sensitive() = true;
	else
		notificationsFrame1.property_sensitive() = false;
}

NotificationsWindow::~NotificationsWindow()
{
}

QueueWindow::QueueWindow(OptionsTab * oT):
	OptionsWindowsBase(_("Queue")),
	activeDownloadsQueueLabel(_("Active torrents [0: without limit]")),
	maxDownloadsQueueLabel(_("Max torrents [0: without limit]")),
	activeSharedQueueLabel(_("Active torrents [0: without limit]"))
{
	// Only show the scrollbars when they are necessary:
	set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	// Buttons
	activeDownloadsQueueButton.set_increments(1, 1);
	activeDownloadsQueueButton.set_range(0, 100);
	activeDownloadsQueueButton.set_numeric();

	maxDownloadsQueueButton.set_increments(1, 1);
	maxDownloadsQueueButton.set_range(0, 100);
	maxDownloadsQueueButton.set_numeric();

	activeSharedQueueButton.set_increments(1, 1);
	activeSharedQueueButton.set_range(0, 100);
	activeSharedQueueButton.set_numeric();

	// Downloads Frame
	Gtk::Table *downloadsTable = Gtk::manage(new Gtk::Table(2, 2, false));
	Gtk::Frame *downloadsFrame = Gtk::manage(new Gtk::Frame(_("Downloads queue")));
	Gtk::HBox *downloadsRow = Gtk::manage(new Gtk::HBox());

	downloadsFrame->set_shadow_type(Gtk::SHADOW_OUT);

	activeDownloadsQueueLabel.property_xalign() = 0;
	downloadsTable->attach(activeDownloadsQueueLabel, 0, 1, 0, 1, Gtk::FILL, Gtk::SHRINK, 5, 5);
	downloadsTable->attach(activeDownloadsQueueButton, 1, 2, 0, 1, Gtk::FILL, Gtk::SHRINK, 5, 5);

	maxDownloadsQueueLabel.property_xalign() = 0;
	downloadsTable->attach(maxDownloadsQueueLabel, 0, 1, 1, 2, Gtk::FILL, Gtk::SHRINK, 5, 5);
	downloadsTable->attach(maxDownloadsQueueButton, 1, 2, 1, 2, Gtk::FILL, Gtk::SHRINK, 5, 5);

	downloadsFrame->add(*downloadsTable);
	downloadsRow->pack_start(*downloadsFrame, Gtk::PACK_EXPAND_WIDGET, 5);

	// Shares Frame
	Gtk::Table *sharesTable = Gtk::manage(new Gtk::Table(2, 2, false));
	Gtk::Frame *sharesFrame = Gtk::manage(new Gtk::Frame(_("Shares queue")));
	Gtk::HBox *sharesRow = Gtk::manage(new Gtk::HBox());

	sharesFrame->set_shadow_type(Gtk::SHADOW_OUT);

	activeSharedQueueLabel.property_xalign() = 0;
	sharesTable->attach(activeSharedQueueLabel, 0, 1, 0, 1, Gtk::FILL, Gtk::SHRINK, 5, 5);
	sharesTable->attach(activeSharedQueueButton, 1, 2, 0, 1, Gtk::FILL, Gtk::SHRINK, 5, 5);

	sharesFrame->add(*sharesTable);
	sharesRow->pack_start(*sharesFrame, Gtk::PACK_EXPAND_WIDGET, 5);

	// Signals
	activeDownloadsQueueButton.signal_value_changed().connect(sigc::mem_fun(*this, &QueueWindow::activeDownloadsQueueButtonChanged));
	activeDownloadsQueueButton.signal_value_changed().connect(sigc::mem_fun(oT, &OptionsTab::setActiveDownloadsConfigValue));

	maxDownloadsQueueButton.signal_value_changed().connect(sigc::mem_fun(*this, &QueueWindow::maxDownloadsQueueButtonChanged));
	maxDownloadsQueueButton.signal_value_changed().connect(sigc::mem_fun(oT, &OptionsTab::setMaxDownloadsConfigValue));

	activeSharedQueueButton.signal_value_changed().connect(sigc::mem_fun(oT, &OptionsTab::setActiveSharesConfigValue));

	// Main VBox
	vbox.pack_start(OptionsWindowsBase::headerText, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(*downloadsRow, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(*sharesRow, Gtk::PACK_SHRINK , 5);

	add(vbox);
}

void QueueWindow::activeDownloadsQueueButtonChanged()
{
	if ((activeDownloadsQueueButton.get_value() > maxDownloadsQueueButton.get_value()) && maxDownloadsQueueButton.get_value_as_int())
		maxDownloadsQueueButton.set_value(activeDownloadsQueueButton.get_value());
}

void QueueWindow::maxDownloadsQueueButtonChanged()
{
	if (!maxDownloadsQueueButton.get_value_as_int()) return;
	else if (maxDownloadsQueueButton.get_value() < activeDownloadsQueueButton.get_value())
		activeDownloadsQueueButton.set_value(maxDownloadsQueueButton.get_value());
}

QueueWindow::~QueueWindow()
{
}

IpFilterWindow::IpFilterWindow(OptionsTab * oT):
	OptionsWindowsBase(_("Ip Filter")),
	activateIpFilterCheckButton(_("Use ip filtering")),
	ipFilterFileLabel(_("Load an Ip filter File ")),
	addIpButton(_("Add")),
	removeIpButton(_("Remove"))
{
	// Only show the scrollbars when they are necessary:
	set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	Gtk::HBox *ipFilterFrameRow = Gtk::manage(new Gtk::HBox(false, 5));
	activateIpFilterBox.pack_start(activateIpFilterCheckButton, Gtk::PACK_SHRINK, 5);

	activateIpFilterCheckButton.set_active();
	activateIpFilterCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &IpFilterWindow::setIpFilterWindowSensivity) );
	activateIpFilterCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setActivateIpFilterConfigValue) );

	ipFilterFileSelectButtonImage.set(Gtk::Stock::DIRECTORY, Gtk::IconSize(1));
	ipFilterFileSelectButton.set_image(ipFilterFileSelectButtonImage);

	ipFilterFileRow.pack_start(ipFilterFileLabel, Gtk::PACK_SHRINK , 5);
	ipFilterFileRow.pack_start(ipFilterFileSelectButton, Gtk::PACK_SHRINK , 5);

	ipFilterFrame.set_shadow_type(Gtk::SHADOW_OUT);
	ipFilterFrame.set_label(_("Blocked Ips"));

	//Ip Filter Tree View
	ipFilterWindow.add(ipFilterTreeView);
	ipFilterWindow.set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	// Create the Tree model:
	refTreeModel = Gtk::ListStore::create(ipFilterColumns);
	ipFilterTreeView.set_model(refTreeModel);
	ipFilterTreeView.set_rules_hint();

	// Add the TreeView's view columns:
	ipFilterTreeView.append_column(_("First"), ipFilterColumns.m_first_ip);
	ipFilterTreeView.append_column(_("Last"), ipFilterColumns.m_last_ip);

	for(guint i = 0; i < 2; i++)
	{
		Gtk::TreeView::Column* pColumn = ipFilterTreeView.get_column(i);
		Gtk::CellRenderer* cColumn = ipFilterTreeView.get_column_cell_renderer(i);

		pColumn->set_resizable();

		switch(i)
		{
			case 0:
				pColumn->set_sort_column(ipFilterColumns.m_first_ip);
				break;
			case 1:
				pColumn->set_sort_column(ipFilterColumns.m_last_ip);
				break;
		}
	}

	ipFilterFrame.add(ipFilterWindow);

	// Signals
	ipFilterFileSelectButton.signal_clicked().connect( sigc::mem_fun(oT, &OptionsTab::selectIpFilterFile) );
	addIpButton.signal_clicked().connect( sigc::mem_fun(oT, &OptionsTab::addIpFilter) );
	removeIpButton.signal_clicked().connect( sigc::mem_fun(oT, &OptionsTab::removeSelectedIpFilter) );

	buttonsHBox.pack_start(spacer, Gtk::PACK_EXPAND_WIDGET , 5);
	buttonsHBox.pack_start(addIpButton, Gtk::PACK_SHRINK , 5);
	buttonsHBox.pack_start(removeIpButton, Gtk::PACK_SHRINK , 5);

	ipFilterFrameRow->pack_start(ipFilterFrame, Gtk::PACK_EXPAND_WIDGET , 5);

	vbox.pack_start(OptionsWindowsBase::headerText, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(activateIpFilterBox, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(ipFilterFileRow, Gtk::PACK_SHRINK , 5);
	vbox.pack_start(*ipFilterFrameRow, Gtk::PACK_EXPAND_WIDGET , 5);
	vbox.pack_start(buttonsHBox, Gtk::PACK_SHRINK , 5);

	add(vbox);

	updateIpFilterData();
}

void IpFilterWindow::updateIpFilterData()
{
	Gtk::TreeModel::Row row;
	SessionManagerPtr ssm = Core::get_session_manager();
	ips_v4 ips;

	ips = ssm->get_ip_filter()->get_filtered_ips_v4();

	refTreeModel->clear();
	for(int i=0;i < ips.size(); i++)
	{
		if (ips[i].flags == libtorrent::ip_filter::blocked)
		{
			row = *(refTreeModel->append());

			row[ipFilterColumns.m_first_ip] = ips[i].first.to_string() ;
			row[ipFilterColumns.m_last_ip] = ips[i].last.to_string();
		}
	}
}

bool IpFilterWindow::isRowSelected()
{
	Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = ipFilterTreeView.get_selection();
	Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();

	if (iter) return true;

	return false;
}

Glib::ustring IpFilterWindow::getFirstSelectedIp()
{
	Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = ipFilterTreeView.get_selection();
	Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();

	if(iter)
	{ //If anything is selected
		Gtk::TreeModel::Row row = *iter;
		return row[ipFilterColumns.m_first_ip];
	}

	return Glib::ustring("");
}

Glib::ustring IpFilterWindow::getLastSelectedIp()
{
	Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = ipFilterTreeView.get_selection();
	Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();

	if(iter)
	{ //If anything is selected
		Gtk::TreeModel::Row row = *iter;
		return row[ipFilterColumns.m_last_ip];
	}

	return Glib::ustring("");
}

void IpFilterWindow::setIpFilterWindowSensivity()
{
	if ( activateIpFilterCheckButton.get_active() )
	{
		ipFilterWindow.property_sensitive () = true;
		ipFilterFileRow.property_sensitive () = true;
		buttonsHBox.property_sensitive () = true;
	}
	else
	{
		ipFilterWindow.property_sensitive () = false;
		ipFilterFileRow.property_sensitive () = false;
		buttonsHBox.property_sensitive () = false;
	}
}

IpFilterWindow::~IpFilterWindow()
{
}

SearchWindow::SearchWindow(OptionsTab * oT):
	OptionsWindowsBase(_("Search")),
	btjunkieCheckButton("Btjunkie"),
	isohuntCheckButton("Isohunt"),
	piratebayCheckButton("Piratebay"),
	mininovaCheckButton("Mininova"),
	torrentreactorCheckButton("Torrentreactor"),
	filterResultsWithoutSeeders(_("Filter results without seeders"))
{
	// Only show the scrollbars when they are necessary:
	set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	Gtk::Frame *engines_frame = Gtk::manage(new Gtk::Frame(_("Search engines")));
	Gtk::Table *engines_table = Gtk::manage(new Gtk::Table(2, 3, false));
	Gtk::Label *maximum_label = Gtk::manage(new Gtk::Label(_("Maximum number of results [0: without limit]")));
	Gtk::HBox *engines_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::HBox *results_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::HBox *filter_row = Gtk::manage(new Gtk::HBox(false, 5));

	engines_frame->set_shadow_type(Gtk::SHADOW_OUT);

	engines_table->attach(btjunkieCheckButton, 0, 1, 0, 1, Gtk::FILL, Gtk::SHRINK, 5, 5);
	engines_table->attach(piratebayCheckButton, 1, 2, 0, 1, Gtk::FILL, Gtk::SHRINK, 5, 5);
	engines_table->attach(isohuntCheckButton, 2, 3, 0, 1, Gtk::FILL, Gtk::SHRINK, 5, 5);

	engines_table->attach(mininovaCheckButton, 0, 1, 1, 2, Gtk::FILL, Gtk::SHRINK, 5, 5);
	engines_table->attach(torrentreactorCheckButton, 1, 2, 1, 2, Gtk::FILL, Gtk::SHRINK, 5, 5);

	maxSearches.set_increments(1, 1);
	maxSearches.set_range(0, 2000);
	maxSearches.set_numeric();

	engines_frame->add(*engines_table);

	// Signals
	btjunkieCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setBtjunkieConfigValue) );
	isohuntCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setIsohuntConfigValue) );
	piratebayCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setPirateBayConfigValue) );
	mininovaCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setMininovaConfigValue) );
	torrentreactorCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setTorrentreactorConfigValue) );
	filterResultsWithoutSeeders.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setFilterResultsWithoutSeedersConfigValue) );

	maxSearches.signal_value_changed().connect(sigc::mem_fun(oT, &OptionsTab::setMaxSearchConfigValue));

	results_row->pack_start(*maximum_label, Gtk::PACK_SHRINK, 5);
	results_row->pack_start(maxSearches, Gtk::PACK_SHRINK, 5);

	engines_row->pack_start(*engines_frame, Gtk::PACK_EXPAND_WIDGET, 5);
	filter_row->pack_start(filterResultsWithoutSeeders, Gtk::PACK_EXPAND_WIDGET, 5);

	vbox.pack_start(OptionsWindowsBase::headerText, Gtk::PACK_SHRINK, 5);
	vbox.pack_start(*engines_row, Gtk::PACK_SHRINK, 5);
	vbox.pack_start(*filter_row, Gtk::PACK_SHRINK, 5);
	vbox.pack_start(*results_row, Gtk::PACK_SHRINK, 5);
	add(vbox);
}

SearchWindow::~SearchWindow()
{
}

SchedulerWindow::SchedulerWindow(OptionsTab * oT):
	OptionsWindowsBase(_("Scheduler")),
	useSchedulerCheckButton(_("Use scheduler")),
	scheduler_frame(_("Scheduler options")),
	monday(_("Monday")),
	tuesday(_("Tuesday")),
	wednesday(_("Wednesday")),
	thursday(_("Thursday")),
	friday(_("Friday")),
	saturday(_("Saturday")),
	sunday(_("Sunday"))
{
	// Only show the scrollbars when they are necessary:
	set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);

	Gtk::HBox *use_scheduler_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::HBox *hours_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::HBox *days_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::HBox *days_frame_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::HBox *update_button_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::HBox *scheduler_frame_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::VBox *scheduler_box = Gtk::manage(new Gtk::VBox(false, 5));
	Gtk::Table *hours_table = Gtk::manage(new Gtk::Table(8, 3, true));
	Gtk::Frame *hours_frame = Gtk::manage(new Gtk::Frame(_("Hours [Time | Download Speed | Upload Speed]")));
	Gtk::Frame *days_frame = Gtk::manage(new Gtk::Frame(_("Days")));
	Gtk::Button *update_button = Gtk::manage(new Gtk::Button(_("Update")));

	scheduler_frame.set_shadow_type(Gtk::SHADOW_OUT);
	hours_frame->set_shadow_type(Gtk::SHADOW_OUT);
	days_frame->set_shadow_type(Gtk::SHADOW_OUT);


	use_scheduler_row->pack_start(useSchedulerCheckButton, Gtk::PACK_EXPAND_WIDGET, 5);
	hours_row->pack_start(*hours_frame, Gtk::PACK_SHRINK, 5);
	days_row->pack_start(*days_frame, Gtk::PACK_SHRINK, 5);
	update_button_row->pack_start(*update_button, Gtk::PACK_SHRINK, 5);
	scheduler_frame_row->pack_start(scheduler_frame, Gtk::PACK_EXPAND_WIDGET, 5);

	days_frame->add(*days_frame_row);

	days_frame_row->pack_start(monday, Gtk::PACK_SHRINK, 5);
	days_frame_row->pack_start(tuesday, Gtk::PACK_SHRINK, 5);
	days_frame_row->pack_start(wednesday, Gtk::PACK_SHRINK, 5);
	days_frame_row->pack_start(thursday, Gtk::PACK_SHRINK, 5);
	days_frame_row->pack_start(friday, Gtk::PACK_SHRINK, 5);
	days_frame_row->pack_start(saturday, Gtk::PACK_SHRINK, 5);
	days_frame_row->pack_start(sunday, Gtk::PACK_SHRINK, 5);

	hours_frame->add(*hours_table);

	// Hours Table
	// Hour 0
	Gtk::HBox *hour0_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour0_label = Gtk::manage(new Gtk::Label("00:00 - 00:59"));
	downloadHour0.set_increments(1, 5);
	downloadHour0.set_range(0, 20000);
	downloadHour0.set_numeric();
	uploadHour0.set_increments(1, 5);
	uploadHour0.set_range(0, 20000);
	uploadHour0.set_numeric();
	hour0_row->pack_start(*hour0_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour0_row->pack_start(downloadHour0, Gtk::PACK_EXPAND_WIDGET, 0);
	hour0_row->pack_start(uploadHour0, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour0_row, 0, 1, 0, 1, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 1
	Gtk::HBox *hour1_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour1_label = Gtk::manage(new Gtk::Label("01:00 - 01:59"));
	downloadHour1.set_increments(1, 5);
	downloadHour1.set_range(0, 20000);
	downloadHour1.set_numeric();
	uploadHour1.set_increments(1, 5);
	uploadHour1.set_range(0, 20000);
	uploadHour1.set_numeric();
	hour1_row->pack_start(*hour1_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour1_row->pack_start(downloadHour1, Gtk::PACK_EXPAND_WIDGET, 0);
	hour1_row->pack_start(uploadHour1, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour1_row, 0, 1, 1, 2, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 2
	Gtk::HBox *hour2_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour2_label = Gtk::manage(new Gtk::Label("02:00 - 02:59"));
	downloadHour2.set_increments(1, 5);
	downloadHour2.set_range(0, 20000);
	downloadHour2.set_numeric();
	uploadHour2.set_increments(1, 5);
	uploadHour2.set_range(0, 20000);
	uploadHour2.set_numeric();
	hour2_row->pack_start(*hour2_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour2_row->pack_start(downloadHour2, Gtk::PACK_EXPAND_WIDGET, 0);
	hour2_row->pack_start(uploadHour2, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour2_row, 0, 1, 2, 3, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 3
	Gtk::HBox *hour3_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour3_label = Gtk::manage(new Gtk::Label("03:00 - 03:59"));
	downloadHour3.set_increments(1, 5);
	downloadHour3.set_range(0, 20000);
	downloadHour3.set_numeric();
	uploadHour3.set_increments(1, 5);
	uploadHour3.set_range(0, 20000);
	uploadHour3.set_numeric();
	hour3_row->pack_start(*hour3_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour3_row->pack_start(downloadHour3, Gtk::PACK_EXPAND_WIDGET, 0);
	hour3_row->pack_start(uploadHour3, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour3_row, 0, 1, 3, 4, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 4
	Gtk::HBox *hour4_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour4_label = Gtk::manage(new Gtk::Label("04:00 - 04:59"));
	downloadHour4.set_increments(1, 5);
	downloadHour4.set_range(0, 20000);
	downloadHour4.set_numeric();
	uploadHour4.set_increments(1, 5);
	uploadHour4.set_range(0, 20000);
	uploadHour4.set_numeric();
	hour4_row->pack_start(*hour4_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour4_row->pack_start(downloadHour4, Gtk::PACK_EXPAND_WIDGET, 0);
	hour4_row->pack_start(uploadHour4, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour4_row, 0, 1, 4, 5, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 5
	Gtk::HBox *hour5_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour5_label = Gtk::manage(new Gtk::Label("05:00 - 05:59"));
	downloadHour5.set_increments(1, 5);
	downloadHour5.set_range(0, 20000);
	downloadHour5.set_numeric();
	uploadHour5.set_increments(1, 5);
	uploadHour5.set_range(0, 20000);
	uploadHour5.set_numeric();
	hour5_row->pack_start(*hour5_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour5_row->pack_start(downloadHour5, Gtk::PACK_EXPAND_WIDGET, 0);
	hour5_row->pack_start(uploadHour5, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour5_row, 0, 1, 5, 6, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 6
	Gtk::HBox *hour6_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour6_label = Gtk::manage(new Gtk::Label("06:00 - 06:59"));
	downloadHour6.set_increments(1, 5);
	downloadHour6.set_range(0, 20000);
	downloadHour6.set_numeric();
	uploadHour6.set_increments(1, 5);
	uploadHour6.set_range(0, 20000);
	uploadHour6.set_numeric();
	hour6_row->pack_start(*hour6_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour6_row->pack_start(downloadHour6, Gtk::PACK_EXPAND_WIDGET, 0);
	hour6_row->pack_start(uploadHour6, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour6_row, 0, 1, 6, 7, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 7
	Gtk::HBox *hour7_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour7_label = Gtk::manage(new Gtk::Label("07:00 - 07:59"));
	downloadHour7.set_increments(1, 5);
	downloadHour7.set_range(0, 20000);
	downloadHour7.set_numeric();
	uploadHour7.set_increments(1, 5);
	uploadHour7.set_range(0, 20000);
	uploadHour7.set_numeric();
	hour7_row->pack_start(*hour7_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour7_row->pack_start(downloadHour7, Gtk::PACK_EXPAND_WIDGET, 0);
	hour7_row->pack_start(uploadHour7, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour7_row, 0, 1, 7, 8, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 8
	Gtk::HBox *hour8_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour8_label = Gtk::manage(new Gtk::Label("08:00 - 08:59"));
	downloadHour8.set_increments(1, 5);
	downloadHour8.set_range(0, 20000);
	downloadHour8.set_numeric();
	uploadHour8.set_increments(1, 5);
	uploadHour8.set_range(0, 20000);
	uploadHour8.set_numeric();
	hour8_row->pack_start(*hour8_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour8_row->pack_start(downloadHour8, Gtk::PACK_EXPAND_WIDGET, 0);
	hour8_row->pack_start(uploadHour8, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour8_row, 1, 2, 0, 1, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 9
	Gtk::HBox *hour9_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour9_label = Gtk::manage(new Gtk::Label("09:00 - 09:59"));
	downloadHour9.set_increments(1, 5);
	downloadHour9.set_range(0, 20000);
	downloadHour9.set_numeric();
	uploadHour9.set_increments(1, 5);
	uploadHour9.set_range(0, 20000);
	uploadHour9.set_numeric();
	hour9_row->pack_start(*hour9_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour9_row->pack_start(downloadHour9, Gtk::PACK_EXPAND_WIDGET, 0);
	hour9_row->pack_start(uploadHour9, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour9_row, 1, 2, 1, 2, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 10
	Gtk::HBox *hour10_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour10_label = Gtk::manage(new Gtk::Label("10:00 - 10:59"));
	downloadHour10.set_increments(1, 5);
	downloadHour10.set_range(0, 20000);
	downloadHour10.set_numeric();
	uploadHour10.set_increments(1, 5);
	uploadHour10.set_range(0, 20000);
	uploadHour10.set_numeric();
	hour10_row->pack_start(*hour10_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour10_row->pack_start(downloadHour10, Gtk::PACK_EXPAND_WIDGET, 0);
	hour10_row->pack_start(uploadHour10, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour10_row, 1, 2, 2, 3, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 11
	Gtk::HBox *hour11_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour11_label = Gtk::manage(new Gtk::Label("11:00 - 11:59"));
	downloadHour11.set_increments(1, 5);
	downloadHour11.set_range(0, 20000);
	downloadHour11.set_numeric();
	uploadHour11.set_increments(1, 5);
	uploadHour11.set_range(0, 20000);
	uploadHour11.set_numeric();
	hour11_row->pack_start(*hour11_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour11_row->pack_start(downloadHour11, Gtk::PACK_EXPAND_WIDGET, 0);
	hour11_row->pack_start(uploadHour11, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour11_row, 1, 2, 3, 4, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 12
	Gtk::HBox *hour12_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour12_label = Gtk::manage(new Gtk::Label("12:00 - 12:59"));
	downloadHour12.set_increments(1, 5);
	downloadHour12.set_range(0, 20000);
	downloadHour12.set_numeric();
	uploadHour12.set_increments(1, 5);
	uploadHour12.set_range(0, 20000);
	uploadHour12.set_numeric();
	hour12_row->pack_start(*hour12_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour12_row->pack_start(downloadHour12, Gtk::PACK_EXPAND_WIDGET, 0);
	hour12_row->pack_start(uploadHour12, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour12_row, 1, 2, 4, 5, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 13
	Gtk::HBox *hour13_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour13_label = Gtk::manage(new Gtk::Label("13:00 - 13:59"));
	downloadHour13.set_increments(1, 5);
	downloadHour13.set_range(0, 20000);
	downloadHour13.set_numeric();
	uploadHour13.set_increments(1, 5);
	uploadHour13.set_range(0, 20000);
	uploadHour13.set_numeric();
	hour13_row->pack_start(*hour13_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour13_row->pack_start(downloadHour13, Gtk::PACK_EXPAND_WIDGET, 0);
	hour13_row->pack_start(uploadHour13, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour13_row, 1, 2, 5, 6, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 14
	Gtk::HBox *hour14_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour14_label = Gtk::manage(new Gtk::Label("14:00 - 14:59"));
	downloadHour14.set_increments(1, 5);
	downloadHour14.set_range(0, 20000);
	downloadHour14.set_numeric();
	uploadHour14.set_increments(1, 5);
	uploadHour14.set_range(0, 20000);
	uploadHour14.set_numeric();
	hour14_row->pack_start(*hour14_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour14_row->pack_start(downloadHour14, Gtk::PACK_EXPAND_WIDGET, 0);
	hour14_row->pack_start(uploadHour14, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour14_row, 1, 2, 6, 7, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 15
	Gtk::HBox *hour15_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour15_label = Gtk::manage(new Gtk::Label("15:00 - 15:59"));
	downloadHour15.set_increments(1, 5);
	downloadHour15.set_range(0, 20000);
	downloadHour15.set_numeric();
	uploadHour15.set_increments(1, 5);
	uploadHour15.set_range(0, 20000);
	uploadHour15.set_numeric();
	hour15_row->pack_start(*hour15_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour15_row->pack_start(downloadHour15, Gtk::PACK_EXPAND_WIDGET, 0);
	hour15_row->pack_start(uploadHour15, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour15_row, 1, 2, 7, 8, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 16
	Gtk::HBox *hour16_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour16_label = Gtk::manage(new Gtk::Label("16:00 - 16:59"));
	downloadHour16.set_increments(1, 5);
	downloadHour16.set_range(0, 20000);
	downloadHour16.set_numeric();
	uploadHour16.set_increments(1, 5);
	uploadHour16.set_range(0, 20000);
	uploadHour16.set_numeric();
	hour16_row->pack_start(*hour16_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour16_row->pack_start(downloadHour16, Gtk::PACK_EXPAND_WIDGET, 0);
	hour16_row->pack_start(uploadHour16, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour16_row, 2, 3, 0, 1, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 17
	Gtk::HBox *hour17_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour17_label = Gtk::manage(new Gtk::Label("17:00 - 17:59"));
	downloadHour17.set_increments(1, 5);
	downloadHour17.set_range(0, 20000);
	downloadHour17.set_numeric();
	uploadHour17.set_increments(1, 5);
	uploadHour17.set_range(0, 20000);
	uploadHour17.set_numeric();
	hour17_row->pack_start(*hour17_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour17_row->pack_start(downloadHour17, Gtk::PACK_EXPAND_WIDGET, 0);
	hour17_row->pack_start(uploadHour17, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour17_row, 2, 3, 1, 2, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 18
	Gtk::HBox *hour18_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour18_label = Gtk::manage(new Gtk::Label("18:00 - 18:59"));
	downloadHour18.set_increments(1, 5);
	downloadHour18.set_range(0, 20000);
	downloadHour18.set_numeric();
	uploadHour18.set_increments(1, 5);
	uploadHour18.set_range(0, 20000);
	uploadHour18.set_numeric();
	hour18_row->pack_start(*hour18_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour18_row->pack_start(downloadHour18, Gtk::PACK_EXPAND_WIDGET, 0);
	hour18_row->pack_start(uploadHour18, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour18_row, 2, 3, 2, 3, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 19
	Gtk::HBox *hour19_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour19_label = Gtk::manage(new Gtk::Label("19:00 - 19:59"));
	downloadHour19.set_increments(1, 5);
	downloadHour19.set_range(0, 20000);
	downloadHour19.set_numeric();
	uploadHour19.set_increments(1, 5);
	uploadHour19.set_range(0, 20000);
	uploadHour19.set_numeric();
	hour19_row->pack_start(*hour19_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour19_row->pack_start(downloadHour19, Gtk::PACK_EXPAND_WIDGET, 0);
	hour19_row->pack_start(uploadHour19, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour19_row, 2, 3, 3, 4, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 20
	Gtk::HBox *hour20_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour20_label = Gtk::manage(new Gtk::Label("20:00 - 20:59"));
	downloadHour20.set_increments(1, 5);
	downloadHour20.set_range(0, 20000);
	downloadHour20.set_numeric();
	uploadHour20.set_increments(1, 5);
	uploadHour20.set_range(0, 20000);
	uploadHour20.set_numeric();
	hour20_row->pack_start(*hour20_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour20_row->pack_start(downloadHour20, Gtk::PACK_EXPAND_WIDGET, 0);
	hour20_row->pack_start(uploadHour20, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour20_row, 2, 3, 4, 5, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 21
	Gtk::HBox *hour21_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour21_label = Gtk::manage(new Gtk::Label("21:00 - 21:59"));
	downloadHour21.set_increments(1, 5);
	downloadHour21.set_range(0, 20000);
	downloadHour21.set_numeric();
	uploadHour21.set_increments(1, 5);
	uploadHour21.set_range(0, 20000);
	uploadHour21.set_numeric();
	hour21_row->pack_start(*hour21_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour21_row->pack_start(downloadHour21, Gtk::PACK_EXPAND_WIDGET, 0);
	hour21_row->pack_start(uploadHour21, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour21_row, 2, 3, 5, 6, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 22
	Gtk::HBox *hour22_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour22_label = Gtk::manage(new Gtk::Label("22:00 - 22:59"));
	downloadHour22.set_increments(1, 5);
	downloadHour22.set_range(0, 20000);
	downloadHour22.set_numeric();
	uploadHour22.set_increments(1, 5);
	uploadHour22.set_range(0, 20000);
	uploadHour22.set_numeric();
	hour22_row->pack_start(*hour22_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour22_row->pack_start(downloadHour22, Gtk::PACK_EXPAND_WIDGET, 0);
	hour22_row->pack_start(uploadHour22, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour22_row, 2, 3, 6, 7, Gtk::FILL, Gtk::FILL, 5, 5);

	// Hour 23
	Gtk::HBox *hour23_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Label *hour23_label = Gtk::manage(new Gtk::Label("23:00 - 23:59"));
	downloadHour23.set_increments(1, 5);
	downloadHour23.set_range(0, 20000);
	downloadHour23.set_numeric();
	uploadHour23.set_increments(1, 5);
	uploadHour23.set_range(0, 20000);
	uploadHour23.set_numeric();
	hour23_row->pack_start(*hour23_label, Gtk::PACK_EXPAND_WIDGET, 5);
	hour23_row->pack_start(downloadHour23, Gtk::PACK_EXPAND_WIDGET, 0);
	hour23_row->pack_start(uploadHour23, Gtk::PACK_EXPAND_WIDGET, 5);

	hours_table->attach(*hour23_row, 2, 3, 7, 8, Gtk::FILL, Gtk::FILL, 5, 5);

	// Signals
	useSchedulerCheckButton.signal_toggled().connect( sigc::mem_fun(*this, &SchedulerWindow::setFramesSensivity) );
	useSchedulerCheckButton.set_active();

	useSchedulerCheckButton.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setUseSchedulerConfigValue) );
	monday.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setMondayConfigValue) );
	tuesday.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setTuesdayConfigValue) );
	wednesday.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setWednesdayConfigValue) );
	thursday.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setThursdayConfigValue) );
	friday.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setFridayConfigValue) );
	saturday.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setSaturdayConfigValue) );
	sunday.signal_toggled().connect( sigc::mem_fun(oT, &OptionsTab::setSundayConfigValue) );
	downloadHour0.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour0ConfigValue) );
	uploadHour0.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour0ConfigValue) );
	downloadHour1.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour1ConfigValue) );
	uploadHour1.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour1ConfigValue) );
	downloadHour2.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour2ConfigValue) );
	uploadHour2.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour2ConfigValue) );
	downloadHour3.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour3ConfigValue) );
	uploadHour3.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour3ConfigValue) );
	downloadHour4.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour4ConfigValue) );
	uploadHour4.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour4ConfigValue) );
	downloadHour5.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour5ConfigValue) );
	uploadHour5.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour5ConfigValue) );
	downloadHour6.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour6ConfigValue) );
	uploadHour6.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour6ConfigValue) );
	downloadHour7.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour7ConfigValue) );
	uploadHour7.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour7ConfigValue) );
	downloadHour8.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour8ConfigValue) );
	uploadHour8.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour8ConfigValue) );
	downloadHour9.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour9ConfigValue) );
	uploadHour9.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour9ConfigValue) );
	downloadHour10.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour10ConfigValue) );
	uploadHour10.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour10ConfigValue) );
	downloadHour11.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour11ConfigValue) );
	uploadHour11.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour11ConfigValue) );
	downloadHour12.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour12ConfigValue) );
	uploadHour12.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour12ConfigValue) );
	downloadHour13.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour13ConfigValue) );
	uploadHour13.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour13ConfigValue) );
	downloadHour14.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour14ConfigValue) );
	uploadHour14.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour14ConfigValue) );
	downloadHour15.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour15ConfigValue) );
	uploadHour15.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour15ConfigValue) );
	downloadHour16.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour16ConfigValue) );
	uploadHour16.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour16ConfigValue) );
	downloadHour17.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour17ConfigValue) );
	uploadHour17.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour17ConfigValue) );
	downloadHour18.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour18ConfigValue) );
	uploadHour18.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour18ConfigValue) );
	downloadHour19.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour19ConfigValue) );
	uploadHour19.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour19ConfigValue) );
	downloadHour20.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour20ConfigValue) );
	uploadHour20.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour20ConfigValue) );
	downloadHour21.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour21ConfigValue) );
	uploadHour21.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour21ConfigValue) );
	downloadHour22.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour22ConfigValue) );
	uploadHour22.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour22ConfigValue) );
	downloadHour23.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setDownloadHour23ConfigValue) );
	uploadHour23.signal_value_changed().connect( sigc::mem_fun(oT, &OptionsTab::setUploadHour23ConfigValue) );

	update_button->signal_clicked().connect( sigc::mem_fun(oT, &OptionsTab::updateSchedulerConfig) );

	scheduler_box->pack_start(*days_row, Gtk::PACK_SHRINK, 5);
	scheduler_box->pack_start(*hours_row, Gtk::PACK_SHRINK, 5);
	scheduler_box->pack_start(*update_button_row, Gtk::PACK_SHRINK, 5);
	scheduler_frame.add(*scheduler_box);

	// Vbox
	vbox.pack_start(OptionsWindowsBase::headerText, Gtk::PACK_SHRINK, 5);
	vbox.pack_start(*use_scheduler_row, Gtk::PACK_SHRINK, 5);
	vbox.pack_start(*scheduler_frame_row, Gtk::PACK_SHRINK, 5);
	add(vbox);
}

void SchedulerWindow::setFramesSensivity()
{
	if (useSchedulerCheckButton.get_active())
		scheduler_frame.property_sensitive () = true;
	else
		scheduler_frame.property_sensitive () = false;
}

SchedulerWindow::~SchedulerWindow()
{
}
