//           SearchTabResults.cc
//  Sat Oct  4 13:54:00 2008
//  Copyright  2008  Fernando Tarín Morales
//  <icemanf@gmail.com>

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU Library General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA

#include "SearchTabResults.h"
#include "Utils.h"
#include "LoadOneTorrentDialog.h"

#include "../config.h"
#include <glibmm/i18n.h>

#include <gtkmm/frame.h>
#include <gtkmm/alignment.h>
#include <gtkmm/label.h>
#include <gtkmm/button.h>
#include <gtkmm/box.h>

#include "Lince/Core.h" 
#include "Lince/SetupManager.h"

using namespace Lince;

SearchResults::SearchResults(GuiSetupManager *_gsm, const Glib::ustring &_search_terms, const Glib::ustring &_categorie):
	search_terms(_search_terms),
	gsm(_gsm),
	results("0"),
	categorie(_categorie),
	status(_("Waiting"))
{
	Gtk::HBox *buttons_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::HBox *status_row = Gtk::manage(new Gtk::HBox(false, 5));
	Gtk::Frame *search_frame = Gtk::manage(new Gtk::Frame());
	Gtk::Button *download_button = Gtk::manage(new Gtk::Button(_("Download")));
	Gtk::Label *status_label = Gtk::manage(new Gtk::Label(_("<b>Status:</b>")));
	Gtk::Label *results_label = Gtk::manage(new Gtk::Label(_("<b>Results:</b>")));
	Gtk::Alignment *spacer = Gtk::manage(new Gtk::Alignment());
	
	// Status row
	status_label->set_use_markup();
	results_label->set_use_markup();
	status_row->pack_start(*status_label, Gtk::PACK_SHRINK, 5);
	status_row->pack_start(status, Gtk::PACK_SHRINK, 5);
	status_row->pack_start(*spacer, Gtk::PACK_SHRINK, 15);
	status_row->pack_start(*results_label, Gtk::PACK_SHRINK, 5);
	status_row->pack_start(results, Gtk::PACK_SHRINK, 5);
	
	// Results treeview
	Gtk::ScrolledWindow *searchResultsScrolledWindow = Gtk::manage(new Gtk::ScrolledWindow());
	
	searchResultsScrolledWindow->add(searchResultsTreeView);
	searchResultsScrolledWindow->set_policy(Gtk::POLICY_AUTOMATIC, Gtk::POLICY_AUTOMATIC);
	
	searchResultsTreeModel = Gtk::ListStore::create(searchResultsColumns);
	searchResultsTreeView.set_model(searchResultsTreeModel);
	searchResultsTreeView.set_rules_hint();
	
	searchResultsTreeView.append_column(_("Name"), searchResultsColumns.m_name);
	searchResultsTreeView.append_column(_("Size"), searchResultsColumns.m_size);
	searchResultsTreeView.append_column(_("Seeds"), searchResultsColumns.m_seeds);
	searchResultsTreeView.append_column(_("Clients"), searchResultsColumns.m_leechs);
	searchResultsTreeView.append_column(_("Engine"), searchResultsColumns.m_engine);
		
	for(guint i = 0; i < 5; i++){
		Gtk::TreeView::Column* pColumn = searchResultsTreeView.get_column(i);
		
		if (i == 0)
			pColumn->set_expand();
		
		switch(i)
		{
			case 0:
				pColumn->set_sort_column(searchResultsColumns.m_name);
				break;
			case 1:
				pColumn->set_sort_column(searchResultsColumns.size);
				break;
			case 2:
				pColumn->set_sort_column(searchResultsColumns.m_seeds);
				break;
			case 3:
				pColumn->set_sort_column(searchResultsColumns.m_leechs);
				break;
			case 4:
				pColumn->set_sort_column(searchResultsColumns.m_engine);
				break;
		}
			
		pColumn->set_resizable();
		pColumn->set_clickable();
	}
	
	
	// Signals
	download_button->signal_clicked().connect( sigc::mem_fun(*this, &SearchResults::downloadTorrent) );
	searchResultsTreeView.signal_row_activated().connect( sigc::mem_fun(*this, &SearchResults::row_activated) );
	
	// Fill containers
	search_frame->add(*searchResultsScrolledWindow);
	
	buttons_row->pack_start(*download_button, Gtk::PACK_SHRINK, 5);
	
	pack_start(*status_row, Gtk::PACK_SHRINK, 5);
	pack_start(*search_frame, Gtk::PACK_EXPAND_WIDGET, 5);
	pack_start(*buttons_row, Gtk::PACK_SHRINK, 5);
	
	is_running = false;
}

void SearchResults::startSearch()
{
	int stdout_fd, stderr_fd;
	std::vector<std::string> command;
	std::string engines;
	
	command.push_back(build_filename_as_string(ENGINE_DIR, "nova2.py"));
	
	if (gsm->get(GuiSetupManager::USE_BTJUNKIE))
		engines += "btjunkie";
	if (gsm->get(GuiSetupManager::USE_ISOHUNT))
	{
		if (!engines.empty())
			engines +=",";
		engines += "isohunt";
	}
	if (gsm->get(GuiSetupManager::USE_PIRATEBAY))
	{
		if (!engines.empty())
			engines +=",";
		engines += "piratebay";
	}
	if (gsm->get(GuiSetupManager::USE_MININOVA))
	{
		if (!engines.empty())
			engines +=",";
		engines += "mininova";
	}
	if (gsm->get(GuiSetupManager::USE_TORRENTREACTOR))
	{
		if (!engines.empty())
			engines +=",";
		engines += "torrentreactor";
	}
	
	command.push_back(engines);
	command.push_back(categorie);
	
	for (unsigned int i=0; i<search_terms.length(); i++)
	{
		if (search_terms[i] == '+') continue;
		else if (search_terms[i] == ' ') search_terms.replace(i, 1, 1, '+');
	}
		
	command.push_back(search_terms);
	
	try{
		Glib::spawn_async_with_pipes(".", command, Glib::SPAWN_SEARCH_PATH|Glib::SPAWN_DO_NOT_REAP_CHILD,
						sigc::slot<void>(), &pid, NULL, &stdout_fd, &stderr_fd);
	}
	catch(Glib::SpawnError &e){
		std::cout << "Error: " << e.what() << std::endl;
		return;
	}

	is_running = true;
	io_stdout = Glib::IOChannel::create_from_fd(stdout_fd);
	io_stdout->set_flags(Glib::IO_FLAG_NONBLOCK);
	io_stderr = Glib::IOChannel::create_from_fd(stderr_fd);
	io_stderr->set_flags(Glib::IO_FLAG_NONBLOCK);
	
	Glib::signal_child_watch().connect(sigc::mem_fun(*this, &SearchResults::searchFinished), pid);
	Glib::signal_timeout().connect(sigc::mem_fun(*this, &SearchResults::showResults), 500);
	
	status.set_text(_("Searching"));
}

bool SearchResults::showResults()
{
	int i=0;
	Glib::ustring line;
	Glib::IOStatus status = Glib::IO_STATUS_NORMAL;
	
	if (is_running){
		while (status == Glib::IO_STATUS_NORMAL || i < 100)
		{
			try
			{
				status = io_stdout->read_line(line);
			}
			catch(Glib::Error e)
			{
			}
			
			if (line.empty() || status == Glib::IO_STATUS_AGAIN ||
				status == Glib::IO_STATUS_ERROR || status == Glib::IO_STATUS_EOF) 
					break;

			int first, last;
			Glib::ustring link, name, size, seeds, leech, engine_url;
			
			first = last = 0;
        	last = line.find("|", first);
        	link = line.substr(first, last-first);
		
			first = last+1;
        	last = line.find("|", first);
        	name = line.substr(first, last-first);
		
			first = last+1;
        	last = line.find("|", first);
        	size = line.substr(first, last-first);
		
			first = last+1;
        	last = line.find("|", first);
        	seeds = line.substr(first, last-first);
		
			first = last+1;
        	last = line.find("|", first);
        	leech = line.substr(first, last-first);
		
			first = last+1;
        	last = line.find("|", first);
        	engine_url = line.substr(first, last-first);
			
			if (gsm->get(GuiSetupManager::FILTER_RESULTS_WITHOUT_SEEDERS) && !to_int(seeds))
					continue;
		
			addRow(link, name, to_int64(size), to_int(seeds), to_int(leech), engine_url);
			i++;
		
			results.set_text(Glib::ustring::compose("%1", searchResultsTreeModel->children().size()));
			
			if (gsm->get(GuiSetupManager::MAX_NUMBER_OF_SEARCHES) == searchResultsTreeModel->children().size())
			{
				stopSearch();
				break;
			}
		}
	}
	
	return is_running;
}

void SearchResults::stopSearch()
{
	if (is_running)
	{
		if (pid) kill(pid, 9);
		io_stdout->close();
		io_stderr->close();
		is_running = false;
		pid = 0;
		if (searchResultsTreeModel->children().size())
			status.set_text(_("Search stopped"));
		else
			status.set_text(_("No results found"));
	}
}

void SearchResults::searchFinished(int, int)
{
	if (is_running)
	{
		io_stdout->close();
		io_stderr->close();
		is_running = false;
		pid = 0;
		if (searchResultsTreeModel->children().size())
			status.set_text(_("Search finished"));
		else
			status.set_text(_("No results found"));
	}
}

void SearchResults::addRow(const Glib::ustring& link, const Glib::ustring& name, boost::int64_t size,
					int seeds, int leechs, const Glib::ustring& engine)
{
	Gtk::TreeModel::Row row;
	
	row = *(searchResultsTreeModel->append());
	
	row[searchResultsColumns.m_link] = link;
	row[searchResultsColumns.m_name] = name;
	row[searchResultsColumns.m_size] = size_to_string(size);
	row[searchResultsColumns.size] = size;
	row[searchResultsColumns.m_seeds] = seeds;
	row[searchResultsColumns.m_leechs] = leechs;
	row[searchResultsColumns.m_engine] = engine;
}

void SearchResults::row_activated(const Gtk::TreeModel::Path&, Gtk::TreeViewColumn*)
{
	downloadTorrent();
}

void SearchResults::downloadTorrent()
{
	Gtk::TreeIter iter = searchResultsTreeView.get_selection()->get_selected();
	
	if (iter)
	{
		std::string name;
		Glib::ustring url, torrentpathsave;
		Gtk::TreeModel::Row row = *iter;
		
		url = row[searchResultsColumns.m_link];
		
		int fd = Glib::file_open_tmp(name, "torrent");
		get_http_torrent(url.c_str(), name);
		close(fd);
		
		std::vector<int> priorities;
		std::vector<std::string> newfilenames;
		
		if (gsm->get(GuiSetupManager::USE_DEFAULT_FILE_DIRECTORY) && 
		!gsm->get(GuiSetupManager::DEFAULT_FILE_DIRECTORY).empty())
		{
			torrentpathsave = gsm->get(GuiSetupManager::DEFAULT_FILE_DIRECTORY);
		}
		else
			torrentpathsave = gsm->get(GuiSetupManager::LAST_SAVE_DIRECTORY);
	
		if (torrentpathsave.empty()) torrentpathsave = getcwd(NULL, 0);
		
		SetupManagerPtr sm = Core::get_setup_manager();
		LoadOneTorrentDialog ld(sm->get(SetupManager::STORAGE_TYPE), name, 
									 &torrentpathsave, &priorities, &newfilenames);
		
		int result = ld.run();
		
		if (result == Gtk::RESPONSE_CANCEL || result == Gtk::RESPONSE_DELETE_EVENT) 
			return;
		
		gsm->set(GuiSetupManager::LAST_SAVE_DIRECTORY, torrentpathsave);
		sm->set(SetupManager::LAST_SAVE_DIRECTORY, torrentpathsave);
		
		boost::filesystem::path p = name;
		Utils::addTorrentToSession(p, torrentpathsave,
							priorities, newfilenames, false);
	}
}

SearchResults::~SearchResults()
{
	if (is_running)
		stopSearch();
}

SearchTabResults::SearchTabResults(GuiSetupManager *_gsm, const Glib::ustring &_search_terms, const Glib::ustring &_categorie):
	search_terms(_search_terms),
	categorie(_categorie),
	BaseTab(_search_terms),
	gsm(_gsm),
	search_results(_gsm, _search_terms, _categorie)
{
}

SearchTabResults::SearchTabResults(const SearchTabResults &t):
	search_terms(t.search_terms),
	BaseTab(t.search_terms),
	gsm(t.gsm),
	search_results(t.gsm, t.search_terms, t.categorie)
{
}

SearchTabResults::~SearchTabResults()
{
}
