/***************************************************************************
 *            StatisticsTab.h
 *
 *  Sat Dec 22 14:14:47 2007
 *  Copyright  2007  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#ifndef _STATISTICSTAB_H
#define _STATISTICSTAB_H

#include "BaseTab.h"
#include "GraphStatsTab.h"
#include "DataStatsTab.h"

class StatisticsTab : public BaseTab, public Gtk::Notebook{
	private:
		GraphStatsTab graphTab;
		DataStatsTab dataTab;

		bool isGraphStatsShown;

	public:
		StatisticsTab();
		~StatisticsTab();

		/* Graph Tab */
		void showGraphStats(bool);

		void setMaxUploadSpeed(float speed) { graphTab.setMaxUploadSpeed(speed); };
		void setMaxDownloadSpeed(float speed) { graphTab.setMaxDownloadSpeed(speed); };

		void addCurrentUploadSpeed(float value) { graphTab.addCurrentUploadSpeed(value); };
		void addCurrentDownloadSpeed(float value) { graphTab.addCurrentDownloadSpeed(value); };

		void updateGraphics(void) { graphTab.updateGraphics(); };

		/* Data tab */
		void updateDownloadSpeed(float value) { dataTab.updateDownloadSpeed(value); };
		void updateDownloadSession(float value) { dataTab.updateDownloadSession(value); };
		void updateDownloadTotal(int64_t value) { dataTab.updateDownloadTotal(value); };

		void updateUploadSpeed(float value) { dataTab.updateUploadSpeed(value); };
		void updateUploadSession(float value) { dataTab.updateUploadSession(value); };
		void updateUploadTotal(int64_t value) { dataTab.updateUploadTotal(value); };

		void updateShareSession() { dataTab.updateShareSession(); };
		void updateShareTotal() { dataTab.updateShareTotal(); };

		void updateTimeSession(double value) { dataTab.updateTimeSession(value); };
		void updateTimeTotal(double value) { dataTab.updateTimeTotal(value); };

		void updateCurrentDownloadingSize(int64_t value) { dataTab.updateCurrentDownloadingSize(value); };
		void updateTotalDownloadingSize(int64_t value) { dataTab.updateTotalDownloadingSize(value); };
		void updateDownloadingRatio() { dataTab.updateDownloadingRatio(); };
		void updateTotalSharedSize(int64_t value) { dataTab.updateTotalSharedSize(value); };

		virtual Gtk::Widget * getTabChild(void) { return this; };
};

#endif /* _STATISTICSTAB_H */


