/***************************************************************************
 *            TorrentsWindow.h
 *
 *  Sat Jan 27 14:19:23 2007
 *  Copyright  2007  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */

#ifndef _TORRENTSWINDOW_H
#define _TORRENTSWINDOW_H

#include <gtkmm.h>

#include "libtorrent/session.hpp"
#include "MenuBar.h"
#include "Utils.h"

#include "Lince/Torrent.h"

class TorrentsTab;
class MainWindow;
class MenuBar;

enum TorrentsWindowType { DOWNLOADING_TORRENTS, DOWNLOADED_TORRENTS };

class TorrentsWindow: public Gtk::ScrolledWindow
{
	private:
		Utils::TorrentModelColumns torrentsColumns;
		TorrentsWindowType torrentsWindowType;
		MenuBar * menuBar;
		TorrentsTab * torrentsTab;

		bool isCtrlKeyPressed;

		// Rows
		Gtk::TreeView torrentsTreeView;
		Glib::RefPtr<Gtk::ListStore> refTreeModel;
		Glib::RefPtr<Gtk::TreeSelection> refTreeSelection;

		Glib::RefPtr<Gtk::ActionGroup> m_refActionGroup;

		// Show columns_autosize
		Glib::RefPtr<Gtk::ToggleAction> showID;
		Glib::RefPtr<Gtk::ToggleAction> showHealth;
		Glib::RefPtr<Gtk::ToggleAction> showName;
		Glib::RefPtr<Gtk::ToggleAction> showSize;
		Glib::RefPtr<Gtk::ToggleAction> showRest;
		Glib::RefPtr<Gtk::ToggleAction> showDownloaded;
		Glib::RefPtr<Gtk::ToggleAction> showUploaded;
		Glib::RefPtr<Gtk::ToggleAction> showRatio;
		Glib::RefPtr<Gtk::ToggleAction> showPercentage;
		Glib::RefPtr<Gtk::ToggleAction> showSeeders;
		Glib::RefPtr<Gtk::ToggleAction> showDownloaders;
		Glib::RefPtr<Gtk::ToggleAction> showStatus;
		Glib::RefPtr<Gtk::ToggleAction> showDSpeed;
		Glib::RefPtr<Gtk::ToggleAction> showUSpeed;
		Glib::RefPtr<Gtk::ToggleAction> showETA;

		// Row Menu
		Glib::RefPtr<Gtk::UIManager> rm_refUIManager;
		Gtk::Menu* rm_pMenuPopup;

		// Menu
		Glib::RefPtr<Gtk::UIManager> nsm_refUIManager;
		Gtk::Menu* nsm_pMenuPopup;

		// Health images
		enum Health { GOOD, BAD, FIREWALLED, NOTRACKER };
		Glib::RefPtr<Gdk::Pixbuf> goodHealth, noTrackerHealth;
		Glib::RefPtr<Gdk::Pixbuf> badHealth, firewalledHealth;

		void setMenuSensitivity();

		void updateRow(Lince::Torrent *, Gtk::TreeModel::Row&);

		void mouse_button_press_event(GdkEventButton* event);
		void key_press_event(GdkEventKey* event);
		void key_release_event(GdkEventKey* event);
		bool focus_in_event(GdkEventFocus*);

		void hideColumn(int);

	public:
		TorrentsWindow(MainWindow*, MenuBar*, TorrentsTab*, TorrentsWindowType);
		~TorrentsWindow();

		int torrentsSelected(void);
		bool getSelectedTorrent(int &);
		bool getSelectedTorrents(std::vector<int>&);

		int getSelectedRow(void);
		void setSelectedRow(int position);
		void removeGivenRow(int tId);

		void addRow(Lince::Torrent *t);
		void updateRows(void);
		void unselectRows(void);
		void clearRows(void) { refTreeModel->clear(); };

		bool isSorted();
		void autoSizeColums(void) { torrentsTreeView.columns_autosize(); };
};

#endif /* _TORRENTSWINDOW_H */
