/***************************************************************************
 *            XMLWriter.h
 *
 *  Sun Jan  7 16:55:36 2007
 *  Copyright  2007  Fernando Tarín Morales
 *  icemanf@gmail.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */
 
#ifndef _XMLWRITER_H
#define _XMLWRITER_H

extern "C"
{
#include <libxml/xmlwriter.h>
}

#include <glibmm/ustring.h>

class XMLWriter{
	public:
		XMLWriter();
		XMLWriter(const Glib::ustring &);
		~XMLWriter();
	
		void setFileName(const Glib::ustring & fname) { filename = fname; };
		const Glib::ustring & getFileName(void) const { return filename; };
		
		void setFormatFile(bool value) { formatfile=value; xmlTextWriterSetIndent(writer, (int) formatfile); };
		void setIndentString(const Glib::ustring & indentstring) { xmlTextWriterSetIndentString(writer, (xmlChar *) indentstring.c_str()); };
		bool startDocument(const Glib::ustring &);
		bool startElement(const Glib::ustring &);
		bool writeElement(const Glib::ustring &, const Glib::ustring &);
		bool addAttribute(const Glib::ustring &, const Glib::ustring &);
		bool addElementData(const Glib::ustring &);
		bool addElementData(int);
		bool addElementData(float);
		bool addElementData(int64_t);
		bool addComment(const Glib::ustring &);
		bool endElement(void);
		bool endDocument(void);
	
	private:
		bool initializeWriter(void);
		bool isWriterInitialized(void) const { return !(writer == NULL); };
		void freeWriter(void) { xmlFreeTextWriter(writer); writer = NULL; };
		
		bool checkUtf8(const Glib::ustring &);
				
		Glib::ustring filename;
		
		bool formatfile;
		xmlTextWriterPtr writer;
		Glib::ustring encoding;
};

#endif /* _XMLWRITER_H */
